# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestEnumerationItem(aeidon.TestCase):

    def setup_method(self, method):
        self.item = aeidon.EnumerationItem(0, "test", object())

    def test___bool__(self):
        assert self.item

    def test___cmp____equal(self):
        if not aeidon.debug: return
        assert self.item == self.item
        assert self.item == 0

    def test___cmp____value_error(self):
        if not aeidon.debug: return
        other = aeidon.EnumerationItem(0, "rest", object())
        self.raises(ValueError, cmp, self.item, other)
        self.raises(ValueError, cmp, self.item, "xxx")

    def test___str__(self):
        assert str(self.item) == "test"


class TestEnumeration(aeidon.TestCase):

    def setup_method(self, method):
        self.fruits = aeidon.Enumeration()
        self.fruits.APPLE = aeidon.EnumerationItem()
        self.fruits.APPLE.size = 10
        self.fruits.MANGO = aeidon.EnumerationItem()
        self.fruits.MANGO.size = 15

    def test___contains____different(self):
        if not aeidon.debug: return
        item = aeidon.EnumerationItem(0, "test", object())
        assert not item in self.fruits

    def test___contains____int(self):
        if not aeidon.debug: return
        assert 0 in self.fruits
        assert 1 in self.fruits

    def test___contains____item(self):
        if not aeidon.debug: return
        assert self.fruits.APPLE in self.fruits
        assert self.fruits.MANGO in self.fruits

    def test___delattr__(self):
        value = self.fruits.MANGO
        delattr(self.fruits, "MANGO")
        assert not hasattr(self.fruits, "MANGO")
        assert not value in self.fruits

    def test___setattr__(self):
        assert self.fruits.APPLE == 0
        assert self.fruits.MANGO == 1
        assert self.fruits.APPLE.name == "APPLE"
        assert self.fruits.MANGO.name == "MANGO"
        assert self.fruits.APPLE.size == 10
        assert self.fruits.MANGO.size == 15

    def test_find_item(self):
        find_item = self.fruits.find_item
        assert find_item("size", 10) == self.fruits.APPLE
        assert find_item("size", 15) == self.fruits.MANGO

    def test_find_item__value_error(self):
        find_item = self.fruits.find_item
        self.raises(ValueError, find_item, "size", 20)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Searching for and replacing text."""

import aeidon
_ = aeidon.i18n._


class SearchAgent(aeidon.Delegate):

    """Searching for and replacing text.

    :ivar _docs: Sequence of :attr:`aeidon.documents` items
    :ivar _finder: Instance of :class:`aeidon.Finder` used
    :ivar _match_doc: :attr:`aeidon.documents` item of the last match
    :ivar _match_passed: ``True`` if the position of last match has been passed
    :ivar _match_index: Index of the last match
    :ivar _match_span: Start and end positions of the last match
    :ivar _indices: Sequence of target indices or ``None`` for all
    :ivar _wrap: ``True`` to wrap search, ``False`` to stop at the last index

    Searching is done with the help of an instance of :class:`aeidon.Finder`.
    This agent provides for looping over the subtitles and their texts, feeding
    those texts to the finder and raising :exc:`StopIteration` when no more
    matches are found.
    """

    __metaclass__ = aeidon.Contractual

    def __init__(self, master):
        """Initialize a :class:`SearchAgent` object."""
        aeidon.Delegate.__init__(self, master)
        self._docs = None
        self._finder = aeidon.Finder()
        self._indices = None
        self._match_doc = None
        self._match_index = None
        self._match_passed = None
        self._match_span = None
        self._wrap = None
        # Set targets to defaults.
        self.set_search_target()

    def _find(self, index, doc, pos, next):
        """Find pattern starting from given location.

        `pos` can be ``None`` for beginning or end.
        `next` should be ``True`` to find next, ``False`` to find previous.
        Raise :exc:`StopIteration` if no match.
        Return tuple of index, document, match span.
        """
        find = (self._next_in_document if next
                else self._previous_in_document)

        self._match_index = index
        self._match_doc = doc
        self._match_passed = False
        indices = self._indices or self.get_all_indices()
        while True:
            try:
                # Return match in document after location.
                return find(index, doc, pos)
            except ValueError:
                pass
            # Proceed to the next document or raise StopIteration.
            self._match_passed = True
            doc = self._get_document(doc, next)
            index = (min(indices) if next else max(indices))
            pos = None

    def _get_document(self, doc, next):
        """Return the document to proceed to.

        `next` should be ``True`` to find next, ``False`` to find previous.
        Raise :exc:`StopIteration` if nowhere to proceed.
        """
        if len(self._docs) == 1:
            if self._wrap:
                return doc
            raise StopIteration
        if next and (doc == aeidon.documents.MAIN):
            return aeidon.documents.TRAN
        if next and (doc == aeidon.documents.TRAN):
            if self._wrap:
                return aeidon.documents.MAIN
            raise StopIteration
        if (not next) and (doc == aeidon.documents.MAIN):
            if self._wrap:
                return aeidon.documents.TRAN
            raise StopIteration
        if (not next) and (doc == aeidon.documents.TRAN):
            return aeidon.documents.MAIN
        raise ValueError("Invalid document: %s or invalid next: %s"
                         % (repr(doc), repr(next)))

    def _invariant(self):
        for index in self._indices or ():
            assert 0 <= index < len(self.subtitles)

    def _next_in_document(self, index, doc, pos=None):
        """Find the next match in `doc` starting from `pos`.

        `pos` can be ``None`` to start from beginning.
        Raise :exc:`StopIteration` if no matches at all anywhere.
        Raise :exc:`ValueError` if no match in this `doc` after `pos`.
        Return tuple of index, document, match span.
        """
        indices = self._indices or self.get_all_indices()
        for index in range(index, max(indices) + 1):
            text = self.subtitles[index].get_text(doc)
            # Avoid resetting finder's match span.
            if text != self._finder.text:
                self._finder.set_text(text)
            self._finder.pos = 0
            if pos is not None:
                self._finder.pos = pos
            try:
                match_span = self._finder.next()
            except StopIteration:
                # Raise StopIteration if a full loop around all target
                # documents and indices has been made with no matches.
                if doc == self._match_doc:
                    if index == self._match_index:
                        if self._match_passed:
                            raise StopIteration
                pos = None
                continue
            self._match_index = index
            self._match_doc = doc
            self._match_span = match_span
            self._match_passed = False
            return index, doc, match_span
        # Raise ValueError if no match found in this document after position.
        raise ValueError("No more matches in document: %s" % repr(doc))

    def _previous_in_document(self, index, doc, pos=None):
        """Find the previous match in `doc` starting from `pos`.

        `pos` can be ``None`` to start from end.
        Raise :exc:`StopIteration` if no matches at all anywhere.
        Raise :exc:`ValueError` if no match in this `doc` before `pos`.
        Return tuple of index, document, match span.
        """
        indices = self._indices or self.get_all_indices()
        for index in reversed(range(min(indices), index + 1)):
            text = self.subtitles[index].get_text(doc)
            # Avoid resetting finder's match span.
            if text != self._finder.text:
                self._finder.set_text(text)
            self._finder.pos = len(self._finder.text)
            if pos is not None:
                self._finder.pos = pos
            try:
                match_span = self._finder.previous()
            except StopIteration:
                # Raise StopIteration if a full loop around all target
                # documents and indices has been made with no matches.
                if doc == self._match_doc:
                    if index == self._match_index:
                        if self._match_passed:
                            raise StopIteration
                pos = None
                continue
            self._match_index = index
            self._match_doc = doc
            self._match_span = match_span
            self._match_passed = False
            return index, doc, match_span
        # Raise ValueError if no match found in this document after position.
        raise ValueError("No more matches in document: %s" % repr(doc))

    def find_next_require(self, index=None, doc=None, pos=None):
        assert 0 <= (index or 0) < len(self.subtitles)

    def find_next_ensure(self, value, index=None, doc=None, pos=None):
        index, doc, match_span = value
        assert 0 <= index < len(self.subtitles)
        text = self.subtitles[index].get_text(doc)
        assert 0 <= match_span[0] <= len(text)
        assert 0 <= match_span[1] <= len(text)
        assert self._docs

    @aeidon.deco.export
    def find_next(self, index=None, doc=None, pos=None):
        """Find the next match starting from given location.

        `index`, `doc` and `pos` can be ``None`` to start from beginning.
        Raise :exc:`StopIteration` if no matches exist.
        Return tuple of index, document, match span.
        """
        index = (0 if index is None else index)
        doc = (self._docs[0] if doc is None else doc)
        return self._find(index, doc, pos, True)

    def find_previous_require(self, index=None, doc=None, pos=None):
        assert 0 <= (index or 0) < len(self.subtitles)
        if doc is not None:
            text = self.subtitles[index or 0].get_text(doc)
            assert 0 <= (pos or 0) <= len(text)

    def find_previous_ensure(self, value, index=None, doc=None, pos=None):
        index, doc, match_span = value
        assert 0 <= index < len(self.subtitles)
        text = self.subtitles[index].get_text(doc)
        assert 0 <= match_span[0] <= len(text)
        assert 0 <= match_span[1] <= len(text)
        assert self._docs

    @aeidon.deco.export
    def find_previous(self, index=None, doc=None, pos=None):
        """Find the previous match starting from given location.

        `index`, `doc` and `pos` can be ``None`` to start from end.
        Raise :exc:`StopIteration` if no matches exist.
        Return tuple of index, document, match span.
        """
        index = (len(self.subtitles) - 1 if index is None else index)
        doc = (self._docs[-1] if doc is None else doc)
        return self._find(index, doc, pos, False)

    def replace_require(self, register=-1):
        assert 0 <= self._match_index < len(self.subtitles)
        subtitle = self.subtitles[self._match_index]
        text = subtitle.get_text(self._match_doc)
        assert 0 <= self._match_span[0] <= len(text)
        assert 0 <= self._match_span[1] <= len(text)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def replace(self, register=-1):
        """Replace the current match of pattern.

        Raise :exc:`re.error` if bad replacement.
        """
        orig_text = self._finder.text
        self._finder.replace()
        if self._finder.text == orig_text: return
        self.set_text(self._match_index,
                      self._match_doc,
                      self._finder.text,
                      register=register)

        self.set_action_description(register, _("Replacing"))

    @aeidon.deco.export
    @aeidon.deco.revertable
    def replace_all(self, register=-1):
        """Replace all matches of pattern and return amount.

        Raise :exc:`re.error` if bad replacement.
        """
        counts = {}
        for doc in self._docs:
            counts[doc] = 0
            new_indices = []
            new_texts = []
            for index, subtitle in enumerate(self.subtitles):
                text = subtitle.get_text(doc)
                self._finder.set_text(text)
                sub_count = self._finder.replace_all()
                if sub_count > 0:
                    new_indices.append(index)
                    new_texts.append(self._finder.text)
                    counts[doc] += sub_count
            if new_indices:
                self.replace_texts(new_indices,
                                   doc,
                                   new_texts,
                                   register=register)

                self.set_action_description(register, _("Replacing all"))
        if (len(counts.keys()) == 2) and all(counts.values()):
            self.group_actions(register, 2, _("Replacing all"))
        return sum(counts.values())

    @aeidon.deco.export
    def set_search_regex(self, pattern, flags=0):
        """Set the regular expression pattern to find.

        ``DOTALL``, ``MULTILINE`` and ``UNICODE`` are automatically added to
        flags. Raise :exc:`re.error` if bad pattern.
        """
        self._finder.set_regex(unicode(pattern), flags)

    @aeidon.deco.export
    def set_search_replacement(self, replacement):
        """Set the replacement string."""
        self._finder.replacement = unicode(replacement)

    @aeidon.deco.export
    def set_search_string(self, pattern, ignore_case=False):
        """Set the string pattern to find."""
        self._finder.pattern = unicode(pattern)
        self._finder.ignore_case = ignore_case

    def set_search_target_require(self, indices=None, docs=None, wrap=True):
        for index in indices or ():
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    def set_search_target(self, indices=None, docs=None, wrap=True):
        """Set the targets to search in.

        `indices` can be ``None`` to target all subtitles.
        `docs` can be ``None`` to target all documents.
        """
        self._indices = (tuple(indices) if indices else None)
        self._docs = tuple(docs or aeidon.documents)
        self._wrap = wrap



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""GTK+ user interface controller for :class:`aeidon.Project`."""

import aeidon
import atexit
import gaupol
import gtk
import itertools
import os
import pango
_ = aeidon.i18n._

__all__ = ("Application",)


class ApplicationMeta(aeidon.Contractual):

    """Application metaclass with delegated methods added.

    Public methods are added to the class dictionary during :meth:`__new__`
    in order to fool Sphinx (and perhaps other API documentation generators)
    into thinking that the resulting instantiated class actually contains those
    methods, which it does not since the methods are be removed during
    :meth:`Application.__init__`.
    """

    def __new__(meta, class_name, bases, dic):
        new_dict = dic.copy()
        for agent_class_name in gaupol.agents.__all__:
            agent_class = getattr(gaupol.agents, agent_class_name)
            def is_public_method(name):
                return (not name.startswith("_") and
                        callable(getattr(agent_class, name)))

            attr_names = filter(is_public_method, dir(agent_class))
            for attr_name in attr_names:
                new_dict[attr_name] = getattr(agent_class, attr_name)
        return type.__new__(meta, class_name, bases, new_dict)


class Application(aeidon.Observable):

    """GTK+ user interface controller for :class:`aeidon.Project`.

    :ivar _delegations: Dictionary mapping method names to agent methods
    :ivar clipboard: Instance of :class:`aeidon.Clipboard` used
    :ivar counter: Iterator used for naming unsaved documents
    :ivar extension_manager: Instance of :class:`gaupol.ExtensionManager` used
    :ivar framerate_combo: A :class:`gtk.ComboBox` used to select framerate
    :ivar notebook: A :class:`gtk.Notebook` used to hold multiple projects
    :ivar output_window: A :class:`gtk.Window` for external process output
    :ivar pages: List of :class:`gaupol.Page` currently open
    :ivar pattern: Last used search pattern or blank if not used
    :ivar recent_manager: Instance of :class:`gtk.RecentManager` used
    :ivar replacement: Last used search replacement or blank if not used
    :ivar statusbar: A :class:`gtk.Statusbar` used to hold messages
    :ivar uim: Instance of :class:`gtk.UIManager` used
    :ivar video_button: A :class:`gtk.Button` used to select a video file
    :ivar video_toolbar: A :class:`gtk.Toolbar` for video actions
    :ivar window: A :class:`gtk.Window` used to hold all the widgets

    Signals and their arguments for callback functions:
     * ``page-added``: application, page
     * ``page-changed``: application, page
     * ``page-closed``: application, page
     * ``page-switched``: application, page
     * ``pages-reordered``: application, page, number
     * ``quit``: application
     * ``text-assistant-request-pages``: application, assistant
    """

    __metaclass__ = ApplicationMeta

    signals = ("page-added",
               "page-changed",
               "page-closed",
               "page-switched",
               "pages-reordered",
               "quit",
               "text-assistant-request-pages",)

    def __getattr__(self, name):
        """Return method delegated to an agent."""
        return self._delegations[name]

    def __init__(self):
        """Initialize an :class:`Application` object."""
        aeidon.Observable.__init__(self)
        self._delegations = {}
        self.clipboard = aeidon.Clipboard()
        self.counter = itertools.count(1)
        self.extension_manager = gaupol.ExtensionManager(self)
        self.framerate_combo = None
        self.notebook = None
        self.output_window = None
        self.pages = []
        self.pattern = ""
        self.recent_manager = gtk.recent_manager_get_default()
        self.replacement = ""
        self.statusbar = None
        self.uim = None
        self.video_button = None
        self.video_toolbar = None
        self.window = None
        self.x_clipboard = gtk.Clipboard()
        self._init_delegations()
        self._init_gui()
        self.extension_manager.find_extensions()
        self.extension_manager.setup_extensions()
        self.update_gui()
        self.window.show()

    def _finalize_uim_actions(self):
        """Connect UI manager actions to widgets and methods."""
        for name in ("main-safe", "main-unsafe"):
            action_group = self.get_action_group(name)
            for action in action_group.list_actions():
                action.finalize(self)

    def _init_delegations(self):
        """Initialize the delegation mappings."""
        for agent_class_name in gaupol.agents.__all__:
            agent = getattr(gaupol.agents, agent_class_name)(self)
            def is_delegate_method(name):
                value = getattr(agent, name)
                return (callable(value) and
                        hasattr(value, "export") and
                        value.export is True)

            attr_names = filter(is_delegate_method, dir(agent))
            for attr_name in attr_names:
                attr_value = getattr(agent, attr_name)
                if attr_name in self._delegations:
                    raise ValueError("Multiple definitions of %s"
                                     % repr(attr_name))

                self._delegations[attr_name] = attr_value
                # Remove class-level function added by ApplicationMeta.
                if hasattr(self.__class__, attr_name):
                    delattr(self.__class__, attr_name)

    def _init_framerate_combo(self):
        """Intialize the framerate combo box on the video toolbar."""
        self.framerate_combo = gtk.combo_box_new_text()
        for name in (x.label for x in aeidon.framerates):
            self.framerate_combo.append_text(name)
        self.framerate_combo.set_active(gaupol.conf.editor.framerate)
        aeidon.util.connect(self, "framerate_combo", "changed")
        tool_item = gtk.ToolItem()
        tool_item.set_border_width(4)
        tool_item.add(self.framerate_combo)
        self.video_toolbar.insert(tool_item, -1)

    def _init_framerate_label(self):
        """Intialize the framerate label on the video toolbar."""
        label = gtk.Label(_("Framerate:"))
        tool_item = gtk.ToolItem()
        tool_item.set_border_width(4)
        tool_item.add(label)
        self.video_toolbar.insert(tool_item, -1)

    def _init_gui(self):
        """Initialize the user interface."""
        vbox = gtk.VBox()
        self._init_window()
        self._init_uim()
        self._init_menubar(vbox)
        self._init_main_toolbar(vbox)
        self._init_notebook(vbox)
        self._init_video_toolbar(vbox)
        self._init_statusbar(vbox)
        self._init_output_window()
        self.window.add(vbox)
        vbox.show_all()
        self._init_visibilities()
        self.set_menu_notify_events("main-safe")
        self.set_menu_notify_events("main-unsafe")
        self._finalize_uim_actions()

    def _init_main_toolbar(self, vbox):
        """Initialize the main toolbar."""
        self._init_redo_button()
        self._init_undo_button()
        toolbar = self.uim.get_widget("/ui/main_toolbar")
        style = gaupol.conf.application_window.toolbar_style
        if style != gaupol.toolbar_styles.DEFAULT:
            toolbar.set_style(style.value)
        gaupol.conf.connect_notify("application_window", "toolbar_style", self)
        vbox.pack_start(toolbar, False, False, 0)

    def _init_menubar(self, vbox):
        """Initialize the menubar."""
        menubar = self.uim.get_widget("/ui/menubar")
        vbox.pack_start(menubar, False, False, 0)

    def _init_notebook(self, vbox):
        """Initialize the notebook."""
        self.notebook = gtk.Notebook()
        self.notebook.set_scrollable(True)
        self.notebook.set_show_border(False)
        self.notebook.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                                    [("text/uri-list", 0, 0)],
                                    gtk.gdk.ACTION_COPY)

        aeidon.util.connect(self, "notebook", "drag-data-received")
        aeidon.util.connect(self, "notebook", "page-reordered")
        callback = self._on_notebook_switch_page
        self.notebook.connect_after("switch-page", callback)
        vbox.pack_start(self.notebook, True , True , 0)

    def _init_output_window(self):
        """Initialize the output window."""
        self.output_window = gaupol.OutputWindow()
        aeidon.util.connect(self, "output_window", "notify::visible")
        self.output_window.props.visible = gaupol.conf.output_window.show

    def _init_redo_button(self):
        """Initialize the redo button on the main toolbar."""
        redo_button = self.get_tool_item("redo_action")
        redo_button.set_menu(gtk.Menu())
        tip = _("Redo undone actions")
        redo_button.set_arrow_tooltip_text(tip)
        callback = self._on_redo_button_show_menu
        redo_button.connect("show-menu", callback)

    def _init_statusbar(self, vbox):
        """Initialize the statusbar."""
        self.statusbar = gtk.Statusbar()
        self.statusbar.set_has_resize_grip(True)
        vbox.pack_start(self.statusbar, False, False, 0)

    def _init_uim(self):
        """Initialize the UI manager."""
        self.uim = gtk.UIManager()
        safe_group = gtk.ActionGroup("main-safe")
        unsafe_group = gtk.ActionGroup("main-unsafe")
        for name in gaupol.actions.__all__:
            action = getattr(gaupol.actions, name)()
            args = (action, action.accelerator)
            if action.action_group == "main-safe":
                safe_group.add_action_with_accel(*args)
            if action.action_group == "main-unsafe":
                unsafe_group.add_action_with_accel(*args)
        self._init_uim_radio_groups(safe_group)
        self._init_uim_radio_groups(unsafe_group)
        self.uim.insert_action_group(safe_group, 0)
        self.uim.insert_action_group(unsafe_group, 1)
        action_group = gtk.ActionGroup("projects")
        self.uim.insert_action_group(action_group, -1)
        ui_xml_file = os.path.join(aeidon.DATA_DIR, "ui", "ui.xml")
        self.uim.add_ui_from_file(ui_xml_file)
        self.window.add_accel_group(self.uim.get_accel_group())
        path = os.path.join(aeidon.CONFIG_HOME_DIR, "accels.conf")
        if os.path.isfile(path):
            gtk.accel_map_load(path)
        atexit.register(gtk.accel_map_save, path)
        self.uim.ensure_update()

    def _init_uim_radio_groups(self, action_group):
        """Initialize the groups of radio actions in action group."""
        actions = action_group.list_actions()
        actions = [x for x in actions if isinstance(x, gtk.RadioAction)]
        for group in set((x.group for x in actions)):
            instance = None
            for action in (x for x in actions if x.group == group):
                if instance is not None:
                    action.set_group(instance)
                instance = instance or action

    def _init_undo_button(self):
        """Initialize the undo button on the main toolbar."""
        undo_button = self.get_tool_item("undo_action")
        undo_button.set_menu(gtk.Menu())
        tip = _("Undo actions")
        undo_button.set_arrow_tooltip_text(tip)
        callback = self._on_undo_button_show_menu
        undo_button.connect("show-menu", callback)

    def _init_visibilities(self):
        """Initialize visibilities of hideable widgets."""
        conf = gaupol.conf.application_window
        toolbar = self.uim.get_widget("/ui/main_toolbar")
        toolbar.props.visible = conf.show_main_toolbar
        self.video_toolbar.props.visible = conf.show_video_toolbar
        self.statusbar.props.visible = conf.show_statusbar

    def _init_video_button(self):
        """Intialize the video button on the video toolbar."""
        # Let's make this resemble a gtk.FileChooserButton,
        # but not actually be one, because they are slow to instantiate.
        hbox = gtk.HBox(False, 4)
        size = gtk.ICON_SIZE_MENU
        image = gtk.image_new_from_stock(gtk.STOCK_FILE, size)
        hbox.pack_start(image, False, False)
        label = gtk.Label()
        label.props.xalign = 0
        label.set_ellipsize(pango.ELLIPSIZE_END)
        hbox.pack_start(label, True, True)
        hbox.pack_start(gtk.VSeparator(), False, False)
        image = gtk.image_new_from_stock(gtk.STOCK_OPEN, size)
        hbox.pack_start(image, False, False)
        self.video_button = gtk.Button()
        self.video_button.add(hbox)
        self.video_button.set_data("label", label)
        self.video_button.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                                        [("text/uri-list", 0, 0)],
                                        gtk.gdk.ACTION_COPY)

        aeidon.util.connect(self, "video_button", "clicked")
        aeidon.util.connect(self, "video_button", "drag-data-received")
        tool_item = gtk.ToolItem()
        tool_item.set_border_width(4)
        tool_item.set_expand(True)
        tool_item.add(self.video_button)
        self.video_toolbar.insert(tool_item, -1)

    def _init_video_label(self):
        """Intialize the video label on the video toolbar."""
        label = gtk.Label(_("Video file:"))
        tool_item = gtk.ToolItem()
        tool_item.set_border_width(4)
        tool_item.add(label)
        self.video_toolbar.insert(tool_item, -1)

    def _init_video_toolbar(self, vbox):
        """Initialize the video toolbar."""
        self.video_toolbar = gtk.Toolbar()
        self._init_video_label()
        self._init_video_button()
        self._init_framerate_label()
        self._init_framerate_combo()
        vbox.pack_start(self.video_toolbar, False, False, 0)

    def _init_window(self):
        """Initialize the main window."""
        self.window = gtk.Window()
        self.window.set_icon_name("gaupol")
        gtk.window_set_default_icon_name("gaupol")
        self.window.resize(*gaupol.conf.application_window.size)
        self.window.move(*gaupol.conf.application_window.position)
        if gaupol.conf.application_window.maximized:
            self.window.maximize()
        aeidon.util.connect(self, "window", "delete-event")
        aeidon.util.connect(self, "window", "window-state-event")



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Help actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class BrowseWikiDocumentationAction(gaupol.Action):

    """Browse wiki documentation."""

    def __init__(self):
        """Initialize a :class:`BrowseWikiDocumentationAction` object."""
        gaupol.Action.__init__(self, "browse_wiki_documentation")
        self.props.label = _("_Wiki Documentation")
        self.props.stock_id = gtk.STOCK_HELP
        self.props.tooltip = _("Browse wiki documentation")
        self.action_group = "main-safe"


class ReportABugAction(gaupol.Action):

    """Submit a bug report."""

    def __init__(self):
        """Initialize a :class:`ReportABugAction` object."""
        gaupol.Action.__init__(self, "report_a_bug")
        self.props.label = _("_Report A Bug")
        self.props.tooltip = _("Submit a bug report")
        self.action_group = "main-safe"


class ViewAboutDialogAction(gaupol.Action):

    """Show information about Gaupol."""

    def __init__(self):
        """Initialize a :class:`ViewAboutDialogAction` object."""
        gaupol.Action.__init__(self, "view_about_dialog")
        self.props.label = _("_About")
        self.props.stock_id = gtk.STOCK_ABOUT
        self.props.tooltip = _("Show information about Gaupol")
        self.action_group = "main-safe"


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2006-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""MPsub file."""

import aeidon
import re

__all__ = ("MPsub",)


class MPsub(aeidon.SubtitleFile):

    """MPsub file.

    :ivar framerate: :attr:`aeidon.framerates` item for current framerate
    :ivar mode: :attr:`aeidon.modes` item for current native mode

    Instance variables :attr:`framerate` and :attr:`mode` depend on what is
    written on the first line of the file header. They will be determined when
    the file is read and are changed when :meth:`set_header` is called.
    """

    _re_position_line = re.compile(r"^(-?[\d.]+) (-?[\d.]+)\s*$")
    format = aeidon.formats.MPSUB
    mode = aeidon.modes.TIME

    def __init__(self, path, encoding, newline=None):
        """Initialize an :class:`MPsub` object."""
        aeidon.SubtitleFile.__init__(self, path, encoding, newline)
        self.framerate = aeidon.framerates.NONE
        self.mode = aeidon.modes.TIME

    def _get_frames(self, subtitles):
        """Return MPsub-style start and end frames."""
        starts = [x.start_frame for x in subtitles]
        ends = [x.end_frame for x in subtitles]
        for i in reversed(range(1, len(subtitles))):
            ends[i] = ends[i] - starts[i]
            starts[i] = starts[i] - ends[i - 1]
        ends[0] = ends[0] - starts[0]
        return map(str, starts), map(str, ends)

    def _get_positions(self, subtitles):
        """Return MPsub-style start and end positions."""
        if self.mode == aeidon.modes.TIME:
            return self._get_times(subtitles)
        if self.mode == aeidon.modes.FRAME:
            return self._get_frames(subtitles)
        raise ValueError("Invalid mode: %s" % repr(self.mode))

    def _get_times(self, subtitles):
        """Return MPsub-style start and end times."""
        starts = [x.start_seconds for x in subtitles]
        ends = [x.end_seconds for x in subtitles]
        deviation = 0
        # Avoid cumulation of rounding errors by keeping track of deviations
        # and adjusting times so that the deviation stays close to zero.
        for i in reversed(range(1, len(subtitles))):
            real_diff = ends[i] - starts[i]
            ends[i] = round(real_diff - deviation, 2)
            deviation += (ends[i] - real_diff)
            real_diff = starts[i] - ends[i - 1]
            starts[i] = round(real_diff - deviation, 2)
            deviation += (starts[i] - real_diff)
        ends[0] = ends[0] - starts[0]
        to_string = lambda x: ("%.2f" % x).replace("-0.00", "0.00")
        return map(to_string, starts), map(to_string, ends)

    def _read_header(self, lines):
        """Read header and remove its lines."""
        header = ""
        while not self._re_position_line.match(lines[0]):
            header += "\n"
            header += lines.pop(0)
        header = header.lstrip()
        self.set_header(header)

    def _read_lines(self):
        """Read file to a unicoded list of lines."""
        lines = ["\n"]
        for line in aeidon.SubtitleFile._read_lines(self):
            lines.append(line)
            match = self._re_position_line.match(line)
            if match is None: continue
            # Remove blank lines above position lines.
            if not lines[-2].strip():
                lines.pop(-2)
        return lines

    def copy_from(self, other):
        """Copy generic properties from `other`."""
        aeidon.SubtitleFile.copy_from(self, other)
        if self.format == other.format:
            self.mode = other.mode
            self.framerate = other.framerate

    def read(self):
        """Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        subtitles = []
        lines = self._read_lines()
        self._read_header(lines)
        convert = (float if self.mode == aeidon.modes.TIME else int)
        end = convert("0")
        for line in lines:
            match = self._re_position_line.match(line)
            if match is None:
                if subtitles[-1].main_text:
                    subtitles[-1].main_text += "\n"
                subtitles[-1].main_text += line
                continue
            subtitle = self._get_subtitle()
            start = end + convert(match.group(1))
            end = start + convert(match.group(2))
            subtitle.start = start
            subtitle.end = end
            subtitles.append(subtitle)
        return subtitles

    def set_header(self, header):
        """Parse and set header, mode and framerate.

        Raise :exc:`ValueError` if ``FORMAT`` line is invalid.
        """
        mode = aeidon.modes.NONE
        framerates = dict((x.mpsub, x) for x in aeidon.framerates)
        for line in header.split("\n"):
            if line.startswith("FORMAT="):
                mode = line[7:].strip()
        if mode == "TIME":
            self.mode = aeidon.modes.TIME
            self.framerate = aeidon.framerates.NONE
            return setattr(self, "header", header)
        if mode in framerates.keys():
            self.mode = aeidon.modes.FRAME
            self.framerate = framerates[mode]
            return setattr(self, "header", header)
        raise ValueError("Invalid FORMAT line: %s" % repr(header))

    def write_to_file(self, subtitles, doc, fobj):
        """Write `subtitles` from `doc` to `fobj`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        n = self.newline.value
        header = self.header.replace("\n", n)
        fobj.write("%s%s%s" % (header, n, n))
        starts, ends = self._get_positions(subtitles)
        for i, subtitle in enumerate(subtitles):
            if i > 0: fobj.write(n)
            fobj.write("%s %s%s" % (starts[i], ends[i], n))
            text = subtitle.get_text(doc).replace("\n", n)
            fobj.write("%s%s" % (text, n))



# Copyright (C) 2011 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestSpeechRecognitionDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        conf = gaupol.conf.speech_recognition
        conf.acoustic_model = aeidon.temp.create_directory()
        conf.phonetic_dict = aeidon.temp.create()
        conf.lang_model = aeidon.temp.create()
        self.application = self.new_application()
        self.dialog = gaupol.SpeechRecognitionDialog(self.application.window,
                                                     self.application)

        self.dialog.show()

    def test__on_advance_spin_value_changed(self):
        self.dialog._advance_spin.set_value(100)
        self.dialog._advance_spin.set_value(200)
        self.dialog._advance_spin.set_value(300)

    def test__on_default_model_check_toggled(self):
        self.dialog._default_model_check.set_active(True)
        self.dialog._default_model_check.set_active(False)
        self.dialog._default_model_check.set_active(True)

    def test__on_noise_spin_value_changed(self):
        self.dialog._noise_spin.set_value(100)
        self.dialog._noise_spin.set_value(200)
        self.dialog._noise_spin.set_value(300)

    def test__on_response__close(self):
        self.dialog.response(gtk.RESPONSE_CLOSE)

    def test__on_response__help(self):
        self.dialog.response(gtk.RESPONSE_HELP)

    def test__on_revert_button_clicked__advance(self):
        self.dialog._advance_spin.set_value(100)
        advance = self.dialog._advance_spin.get_value_as_int()
        assert advance == 100
        self.dialog._revert_button.clicked()
        advance = self.dialog._advance_spin.get_value_as_int()
        default = gaupol.conf.query_default("speech_recognition",
                                            "advance_length")

        assert advance == default

    def test__on_revert_button_clicked__noise(self):
        self.dialog._noise_spin.set_value(100)
        noise = self.dialog._noise_spin.get_value_as_int()
        assert noise == 100
        self.dialog._revert_button.clicked()
        noise = self.dialog._noise_spin.get_value_as_int()
        default = gaupol.conf.query_default("speech_recognition",
                                            "noise_level")

        assert noise == default

    def test__on_revert_button_clicked__silence(self):
        self.dialog._silence_spin.set_value(100)
        silence = self.dialog._silence_spin.get_value_as_int()
        assert silence == 100
        self.dialog._revert_button.clicked()
        silence = self.dialog._silence_spin.get_value_as_int()
        default = gaupol.conf.query_default("speech_recognition",
                                            "silence_length")

        assert silence == default

    def test__on_silence_spin_value_changed(self):
        self.dialog._silence_spin.set_value(100)
        self.dialog._silence_spin.set_value(200)
        self.dialog._silence_spin.set_value(300)



# Copyright (C) 2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dictionaries mapping enumeration items to UI manager action names.

:var field_actions: Dictionary mapping :attr:`gaupol.formats` to actions
:var framerate_actions: Dictionary mapping :attr:`aeidon.framerates` to actions
:var mode_actions: Dictionary mapping :attr:`aeidon.modes` to actions
"""

import aeidon
import gaupol

__all__ = ("field_actions", "framerate_actions", "mode_actions")


field_actions = {gaupol.fields.NUMBER:    "toggle_number_column",
                 gaupol.fields.START:     "toggle_start_column",
                 gaupol.fields.END:       "toggle_end_column",
                 gaupol.fields.DURATION:  "toggle_duration_column",
                 gaupol.fields.MAIN_TEXT: "toggle_main_text_column",
                 gaupol.fields.TRAN_TEXT: "toggle_translation_text_column",}

framerate_actions = {aeidon.framerates.FPS_23_976: "show_framerate_23_976",
                     aeidon.framerates.FPS_24_000: "show_framerate_24_000",
                     aeidon.framerates.FPS_25_000: "show_framerate_25_000",
                     aeidon.framerates.FPS_29_970: "show_framerate_29_970",}

mode_actions = {aeidon.modes.TIME:  "show_times",
                aeidon.modes.FRAME: "show_frames",}



# Copyright (C) 2007-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk

from ..assistants import CapitalizationPage
from ..assistants import CommonErrorPage
from ..assistants import ConfirmationPage
from ..assistants import HearingImpairedPage
from ..assistants import IntroductionPage
from ..assistants import JoinSplitWordsPage
from ..assistants import LineBreakPage
from ..assistants import LineBreakOptionsPage
from ..assistants import ProgressPage


class TestTextAssistantPage(gaupol.TestCase):

    def setup_method(self, method):
        self.page = gaupol.TextAssistantPage(gtk.Window())


class _TestBuilderPage(gaupol.TestCase):

    # pylint: disable=E1101

    def run__page(self):
        self.window.add(self.page)
        self.window.connect("delete-event", gtk.main_quit)
        self.window.set_default_size(600, 400)
        self.window.show_all()
        gtk.main()


class TestIntroductionPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = IntroductionPage(self.window)
        self.test_populate_tree_view()

    def test__on_all_radio_toggled(self):
        self.page._all_radio.set_active(True)
        self.page._current_radio.set_active(True)
        self.page._selected_radio.set_active(True)

    def test__on_current_radio_toggled(self):
        self.page._all_radio.set_active(True)
        self.page._current_radio.set_active(True)
        self.page._selected_radio.set_active(True)

    def test__on_main_radio_toggled(self):
        self.page._main_radio.set_active(True)
        self.page._tran_radio.set_active(True)
        self.page._main_radio.set_active(True)

    def test__on_selected_radio_toggled(self):
        self.page._all_radio.set_active(True)
        self.page._current_radio.set_active(True)
        self.page._selected_radio.set_active(True)

    def test__on_tran_radio_toggled(self):
        self.page._tran_radio.set_active(True)
        self.page._main_radio.set_active(True)
        self.page._tran_radio.set_active(True)

    def test__on_tree_view_cell_toggled(self):
        store = self.page._tree_view.get_model()
        column = self.page._tree_view.get_column(0)
        renderer = column.get_cell_renderers()[0]
        for i in range(len(store)):
            renderer.emit("toggled", i)
            renderer.emit("toggled", i)
            renderer.emit("toggled", i)

    def test_get_field__main(self):
        self.page._main_radio.set_active(True)
        field = self.page.get_field()
        assert field == gaupol.fields.MAIN_TEXT

    def test_get_field__tran(self):
        self.page._tran_radio.set_active(True)
        field = self.page.get_field()
        assert field == gaupol.fields.TRAN_TEXT

    def test_get_selected_pages(self):
        self.page.get_selected_pages()

    def test_get_target__all(self):
        self.page._all_radio.set_active(True)
        target = self.page.get_target()
        assert target == gaupol.targets.ALL

    def test_get_target__current(self):
        self.page._current_radio.set_active(True)
        target = self.page.get_target()
        assert target == gaupol.targets.CURRENT

    def test_get_target__selected(self):
        self.page._selected_radio.set_active(True)
        target = self.page.get_target()
        assert target == gaupol.targets.SELECTED

    def test_populate_tree_view(self):
        self.page.populate_tree_view((CapitalizationPage(gtk.Window()),
                                      CommonErrorPage(gtk.Window())))


class _TestLocalePage(_TestBuilderPage):

    # pylint: disable=E1101

    def test__on_country_combo_changed(self):
        store = self.page._country_combo.get_model()
        for i in range(len(store)):
            if store[i][0] != gaupol.COMBO_SEPARATOR:
                self.page._country_combo.set_active(i)

    def test__on_language_combo_changed(self):
        store = self.page._language_combo.get_model()
        for i in range(len(store)):
            if store[i][0] != gaupol.COMBO_SEPARATOR:
                self.page._language_combo.set_active(i)

    def test__on_script_combo_changed(self):
        store = self.page._script_combo.get_model()
        for i in range(len(store)):
            if store[i][0] != gaupol.COMBO_SEPARATOR:
                self.page._script_combo.set_active(i)

    def test__on_tree_view_cell_toggled(self):
        store = self.page._tree_view.get_model()
        column = self.page._tree_view.get_column(0)
        renderer = column.get_cell_renderers()[0]
        for i in range(len(store)):
            renderer.emit("toggled", i)
            renderer.emit("toggled", i)
            renderer.emit("toggled", i)

    def test__populate_tree_view(self):
        self.page._populate_tree_view()

    def test_correct_texts(self):
        self.page.correct_texts(self.new_project(),
                                None,
                                aeidon.documents.MAIN)


class TestCapitalizationPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = CapitalizationPage(self.window)


class TestCommonErrorPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = CommonErrorPage(self.window)

    def test__on_human_check_toggled(self):
        self.page._human_check.set_active(True)
        self.page._human_check.set_active(False)
        self.page._human_check.set_active(True)

    def test__on_ocr_check_toggled(self):
        self.page._ocr_check.set_active(True)
        self.page._ocr_check.set_active(False)
        self.page._ocr_check.set_active(True)


class TestHearingImpairedPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = HearingImpairedPage(self.window)


class TestJoinSplitWordsPage(_TestBuilderPage):

    def run__show_error_dialog(self):
        self.page._show_error_dialog("test")

    def setup_method(self, method):
        self.project = self.new_project()
        self.window = gtk.Window()
        self.window.show_all()
        self.page = JoinSplitWordsPage(self.window)

    def test_on_join_check_toggled(self):
        self.page._join_check.set_active(True)
        self.page._join_check.set_active(False)
        self.page._join_check.set_active(True)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__on_language_button_clicked(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.page._language_button.clicked()

    def test_on_split_check_toggled(self):
        self.page._split_check.set_active(True)
        self.page._split_check.set_active(False)
        self.page._split_check.set_active(True)

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.page._show_error_dialog("test")

    def test_correct_texts(self):
        self.page.correct_texts(self.project,
                                None,
                                aeidon.documents.MAIN)


class TestLineBreakPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = LineBreakPage(self.window)

    def test_correct_texts__no_skip(self):
        gaupol.conf.line_break.use_skip_max_length = False
        gaupol.conf.line_break.use_skip_max_lines = False
        return self.test_correct_texts()

    def test_correct_texts__skip(self):
        gaupol.conf.line_break.use_skip_max_length = True
        gaupol.conf.line_break.use_skip_max_lines = True
        return self.test_correct_texts()


class TestLineBreakOptionsPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = LineBreakOptionsPage(self.window)

    def test__on_max_length_spin_value_changed(self):
        self.page._max_length_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.page._max_length_spin.spin(gtk.SPIN_STEP_BACKWARD)

    def test__on_max_lines_spin_value_changed(self):
        self.page._max_lines_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.page._max_lines_spin.spin(gtk.SPIN_STEP_BACKWARD)

    def test__on_max_skip_length_spin_value_changed(self):
        self.page._max_skip_length_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.page._max_skip_length_spin.spin(gtk.SPIN_STEP_BACKWARD)

    def test__on_max_skip_lines_spin_value_changed(self):
        self.page._max_skip_lines_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.page._max_skip_lines_spin.spin(gtk.SPIN_STEP_BACKWARD)

    def test__on_skip_length_check_toggled(self):
        self.page._skip_length_check.set_active(True)
        self.page._skip_length_check.set_active(False)
        self.page._skip_length_check.set_active(True)

    def test__on_skip_lines_check_toggled(self):
        self.page._skip_lines_check.set_active(True)
        self.page._skip_lines_check.set_active(False)
        self.page._skip_lines_check.set_active(True)

    def test__on_skip_unit_combo_changed(self):
        map(self.page._skip_unit_combo.set_active, gaupol.length_units)

    def test__on_unit_combo_changed(self):
        map(self.page._skip_unit_combo.set_active, gaupol.length_units)


class TestProgressPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = ProgressPage(self.window)
        self.page.reset(100)

    def test_bump_progress(self):
        self.page.bump_progress()

    def test_reset(self):
        self.page.reset(100)

    def test_set_progress(self):
        self.page.set_progress(33, 100)
        self.page.set_progress(66, 100)

    def test_set_project_name(self):
        self.page.set_project_name("Test")

    def test_set_task_name(self):
        self.page.set_task_name("Test")


class TestConfirmationPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = gtk.Window()
        self.window.show_all()
        self.page = ConfirmationPage(self.window)
        self.page.application = self.new_application()
        self.page.doc = aeidon.documents.MAIN
        gaupol.conf.preview.use_custom_command = True
        gaupol.conf.preview.custom_command = "echo"
        page = self.page.application.get_current_page()
        page.project.video_path = self.new_subrip_file()
        self.test_populate_tree_view()

    def test__on_mark_all_button_clicked(self):
        self.page._mark_all_button.emit("clicked")
        self.page._mark_all_button.emit("clicked")

    def test__on_preview_button_clicked(self):
        selection = self.page._tree_view.get_selection()
        selection.select_path(0)
        self.page._preview_button.emit("clicked")

    def test__on_remove_check_toggled(self):
        self.page._remove_check.set_active(True)
        self.page._remove_check.set_active(False)
        self.page._remove_check.set_active(True)

    def test__on_tree_view_cell_edited(self):
        store = self.page._tree_view.get_model()
        column = self.page._tree_view.get_column(2)
        renderer = column.get_cell_renderers()[0]
        for i in range(len(store)):
            renderer.emit("edited", i, "test")
            renderer.emit("edited", i, "test")
            renderer.emit("edited", i, "test")

    def test__on_tree_view_cell_toggled(self):
        store = self.page._tree_view.get_model()
        column = self.page._tree_view.get_column(0)
        renderer = column.get_cell_renderers()[0]
        for i in range(len(store)):
            renderer.emit("toggled", i)
            renderer.emit("toggled", i)
            renderer.emit("toggled", i)

    def test__on_tree_view_selection_changed(self):
        selection = self.page._tree_view.get_selection()
        store = self.page._tree_view.get_model()
        for i in range(len(store)):
            selection.select_path(i)

    def test__on_unmark_all_button_clicked(self):
        self.page._unmark_all_button.emit("clicked")
        self.page._unmark_all_button.emit("clicked")

    def test_get_confirmed_changes(self):
        self.page.get_confirmed_changes()

    def test_populate_tree_view(self):
        page = self.page.application.get_current_page()
        changes = [(page, i, x.main_text, x.tran_text) for i, x in
                   enumerate(page.project.subtitles)]

        self.page.populate_tree_view(changes)


class TestTextAssistant(gaupol.TestCase):

    def run__assistant(self):
        self.assistant.show()
        for signal in ("delete-event", "apply", "cancel"):
            self.assistant.connect(signal, gtk.main_quit)
        gtk.main()

    def setup_method(self, method):
        gaupol.conf.editor.use_custom_font = True
        gaupol.conf.editor.custom_font = "monospace"
        gaupol.conf.line_break.max_length = 10
        gaupol.conf.line_break.use_skip_max_length = False
        gaupol.conf.line_break.use_skip_max_lines = False
        gaupol.conf.text_assistant.pages = ["line-break"]
        self.application = self.new_application()
        self.assistant = gaupol.TextAssistant(self.application.window,
                                              self.application)

        self.assistant.show()

    def test__on_apply(self):
        count = self.assistant.get_n_pages()
        indices = range(count)
        # Remove progress page, which is automatically switched
        # to the confirmation page once done with all.
        indices.remove(count - 2)
        map(self.assistant.set_current_page, indices)
        page = self.assistant._confirmation_page
        store = page._tree_view.get_model()
        column = page._tree_view.get_column(2)
        renderer = column.get_cell_renderers()[0]
        for i in range(0, len(store), 2):
            renderer.emit("edited", i, "")
        self.assistant.emit("apply")

    def test__on_cancel(self):
        self.assistant.emit("cancel")

    def test__on_close(self):
        self.assistant.emit("close")



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol


class TestViewAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()

    def test_on_framerate_combo_changed__frames(self):
        self.application.get_action("show_frames").activate()
        for framerate in aeidon.framerates:
            self.application.framerate_combo.set_active(framerate)

    def test_on_framerate_combo_changed__times(self):
        self.application.get_action("show_times").activate()
        for framerate in aeidon.framerates:
            self.application.framerate_combo.set_active(framerate)

    def test_on_output_window_notify_visible(self):
        self.application.output_window.show()
        self.application.output_window.hide()

    def test_on_show_framerate_23_976_changed__frames(self):
        self.application.get_action("show_frames").activate()
        for framerate in aeidon.framerates:
            self.application.get_framerate_action(framerate).activate()

    def test_on_show_framerate_23_976_changed__times(self):
        self.application.get_action("show_times").activate()
        for framerate in aeidon.framerates:
            self.application.get_framerate_action(framerate).activate()

    def test_on_show_times_changed(self):
        for mode in aeidon.modes:
            self.application.get_mode_action(mode).activate()

    def test_on_toggle_duration_column_toggled(self):
        name = "toggle_duration_column"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_end_column_toggled(self):
        name = "toggle_end_column"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_main_text_column_toggled(self):
        name = "toggle_main_text_column"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_main_toolbar_toggled(self):
        name = "toggle_main_toolbar"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_number_column_toggled(self):
        name = "toggle_number_column"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_output_window_toggled(self):
        name = "toggle_output_window"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_start_column_toggled(self):
        name = "toggle_start_column"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_statusbar_activate(self):
        name = "toggle_statusbar"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_translation_text_column_toggled(self):
        name = "toggle_translation_text_column"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()

    def test_on_toggle_video_toolbar_toggled(self):
        name = "toggle_video_toolbar"
        self.application.get_action(name).activate()
        self.application.get_action(name).activate()



# Copyright (C) 2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Internationalization functions.

Functions defined in this module are convenience aliases for functions of the
:mod:`gettext` module with proper initialization and character encodings.
"""

import aeidon
import gettext

__all__ = ("_", "dgettext", "ngettext")

_translation = gettext.translation("gaupol", aeidon.LOCALE_DIR, fallback=True)


def _(message):
    """Return the localized translation of `message`."""
    return _translation.gettext(unicode(message))

def dgettext(domain, message):
    """Return the localized translation of `message` from `domain`."""
    return gettext.dgettext(domain, unicode(message))

def ngettext(singular, plural, n):
    """Return the localized translation of `singular` or `plural`."""
    return _translation.ngettext(unicode(singular), unicode(plural), n)



# Copyright (C) 2007 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Extension delegates of of :class:`gaupol.Application`."""

from .close    import CloseAgent
from .edit     import EditAgent
from .format   import FormatAgent
from .help     import HelpAgent
from .menu     import MenuAgent
from .open     import OpenAgent
from .position import PositionAgent
from .preview  import PreviewAgent
from .save     import SaveAgent
from .search   import SearchAgent
from .text     import TextAgent
from .update   import UpdateAgent
from .util     import UtilityAgent
from .view     import ViewAgent

__all__ = tuple(filter(lambda x: x.endswith("Agent"), dir()))



# Copyright (C) 2006-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""TMPlayer file."""

import aeidon
import re

__all__ = ("TMPlayer",)


class TMPlayer(aeidon.SubtitleFile):

    """TMPlayer file."""

    _re_one_digit_hour = re.compile(r"^-?\d:\d\d:\d\d:")
    _re_two_digit_hour = re.compile(r"^-?\d\d:\d\d:\d\d:")
    format = aeidon.formats.TMPLAYER
    mode = aeidon.modes.TIME

    def __init__(self, path, encoding, newline=None):
        """Initialize an :class:`MPsub` object."""
        aeidon.SubtitleFile.__init__(self, path, encoding, newline)
        self.two_digit_hour = True

    def copy_from(self, other):
        """Copy generic properties from `other` file."""
        aeidon.SubtitleFile.copy_from(self, other)
        if self.format == other.format:
            self.two_digit_hour = other.two_digit_hour

    def read(self):
        """Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        subtitles = [self._get_subtitle()]
        for line in self._read_lines():
            match = self._re_one_digit_hour.search(line)
            if match is not None:
                i = match.span()[1]
                subtitle = self._get_subtitle()
                subtitle.start = ("-0" + line[1:i - 1] + ".000"
                                  if line.startswith("-") else
                                  "0" + line[:i - 1] + ".000")

                subtitles[-1].end = subtitle.start_time
                subtitle.main_text = line[i:].replace("|", "\n")
                subtitles.append(subtitle)
                self.two_digit_hour = False
            match = self._re_two_digit_hour.search(line)
            if match is not None:
                i = match.span()[1]
                subtitle = self._get_subtitle()
                subtitle.start = line[:i - 1] + ".000"
                subtitles[-1].end = subtitle.start_time
                subtitle.main_text = line[i:].replace("|", "\n")
                subtitles.append(subtitle)
                self.two_digit_hour = True
        subtitles.pop(0)
        calc = subtitles[-1].calc
        time = subtitles[-1].start_time
        time = calc.add_seconds_to_time(time, 5)
        subtitles[-1].end = time
        return subtitles

    def write_to_file(self, subtitles, doc, fobj):
        """Write `subtitles` from `doc` to `fobj`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        for subtitle in subtitles:
            start = subtitle.calc.round_time(subtitle.start_time, 0)
            fobj.write("%s:" % (start[:-4] if self.two_digit_hour
                                else "-" + start[2:-4] if start.startswith("-")
                                else start[1:-4]))

            fobj.write(subtitle.get_text(doc).replace("\n", "|"))
            fobj.write(self.newline.value)



# Copyright (C) 2006-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for searching for and replacing text."""

import aeidon
import functools
import gaupol
import gtk
import os
import re
_ = aeidon.i18n._

__all__ = ("SearchDialog",)


def page_changing(function):
    """Decorator for :class:`SearchDialog` methods that edit data."""
    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        args[0]._handle_page_changes = False
        value = function(*args, **kwargs)
        args[0]._handle_page_changes = True
        return value
    return wrapper


class SearchDialog(gaupol.BuilderDialog):

    """Dialog for searching for and replacing text.

    :ivar _handle_page_changes: ``True`` to invalidate search on page changes
    :ivar _match_doc: :attr:`gaupol.documents` item of the last match
    :ivar _match_page: :class:`gaupol.Page` instance of the last match
    :ivar _match_row: Row in :attr:`_match_page` of the last match
    :ivar _match_span: Start, end position of the last match
    :ivar _was_next: ``True`` if the last search was "next", else "previous"
    :ivar patterns: List of patterns previously searched for
    :ivar replacements: List of replacements previously used
    """

    __metaclass__ = aeidon.Contractual

    _widgets = ("all_radio",
                "current_radio",
                "ignore_case_check",
                "main_check",
                "next_button",
                "pattern_combo",
                "previous_button",
                "regex_check",
                "replace_all_button",
                "replace_button",
                "replacement_combo",
                "search_vbox",
                "text_view",
                "tran_check")

    def __init__(self, application):
        """Initialize a :class:`SearchDialog` object."""
        gaupol.BuilderDialog.__init__(self, "search-dialog.ui")
        self._handle_page_changes = True
        self._match_doc = None
        self._match_page = None
        self._match_row = None
        self._match_span = None
        self._pattern_entry = self._pattern_combo.child
        self._replacement_entry = self._replacement_combo.child
        self._was_next = None
        self.application = application
        self.patterns = []
        self.replacements = []
        self._read_history("patterns")
        self._read_history("replacements")
        self._init_text_view()
        self._init_pattern_combo()
        self._init_replacement_combo()
        self._init_values()
        self._init_signal_handlers()
        self._init_sensitivities()
        self._dialog.set_transient_for(None)
        self._dialog.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)

    def _add_pattern_to_history(self):
        """Add current pattern to the pattern combo box."""
        pattern = self._pattern_entry.get_text()
        store = self._pattern_combo.get_model()
        if pattern == self.application.pattern: return
        self.application.pattern = pattern
        self.patterns.insert(0, pattern)
        self.patterns = aeidon.util.get_unique(self.patterns)
        del self.patterns[gaupol.conf.search.max_history:]
        store.clear()
        for pattern in self.patterns:
            store.append((pattern,))
        self._write_history("patterns")
        self.application.update_gui()

    def _add_replacement_to_history(self):
        """Add current replacement to the replacement combo box."""
        replacement = self._replacement_entry.get_text()
        store = self._replacement_combo.get_model()
        if replacement == self.application.replacement: return
        self.application.replacement = replacement
        self.replacements.insert(0, replacement)
        self.replacements = aeidon.util.get_unique(self.replacements)
        del self.replacements[gaupol.conf.search.max_history:]
        store.clear()
        for replacement in self.replacements:
            store.append((replacement,))
        self._write_history("replacements")
        self.application.update_gui()

    def _get_cursor_offset(self, page, row, doc, next):
        """Return current cursor offset of the search or ``None``."""
        if page is not self._match_page: return None
        if row != self._match_row: return None
        if doc != self._match_doc: return None
        return self._match_span[next]

    def _get_fields(self):
        """Return a sequence of fields to search in."""
        fields = []
        if self._main_check.get_active():
            fields.append(gaupol.fields.MAIN_TEXT)
        if self._tran_check.get_active():
            fields.append(gaupol.fields.TRAN_TEXT)
        return tuple(fields)

    def _get_position(self, next):
        """Return current position of the search.

        Raise :exc:`gaupol.Default` if no pages open.
        Return row, document, cursor offset.
        """
        page = self.application.get_current_page()
        gaupol.util.raise_default(page is None)
        rows = page.view.get_selected_rows()
        row = (rows[0] if rows else None)
        if row is None:
            row = (0 if next else len(page.project.subtitles) - 1)
        col = page.view.get_focus()[1]
        doc = None
        if page.view.is_text_column(col):
            doc = page.text_column_to_document(col)
        docs = map(gaupol.util.text_field_to_document,
                   gaupol.conf.search.fields)

        if (doc is None) or (not doc in docs):
            doc = (docs[0] if next else docs[-1])
        pos = self._get_cursor_offset(page, row, doc, next)
        return row, doc, pos

    def _get_target(self):
        """Return :attr:`gaupol.targets` item to search in."""
        if self._current_radio.get_active():
            return gaupol.targets.CURRENT
        if self._all_radio.get_active():
            return gaupol.targets.ALL
        raise ValueError("Invalid target radio state")

    def _init_pattern_combo(self):
        """Initialize the pattern combo box."""
        store = gtk.ListStore(str)
        self._pattern_combo.set_model(store)
        for pattern in self.patterns:
            store.append((pattern,))
        self._pattern_combo.set_text_column(0)

    def _init_replacement_combo(self):
        """Initialize the replacement combo box."""
        store = gtk.ListStore(str)
        self._replacement_combo.set_model(store)
        for replacement in self.replacements:
            store.append((replacement,))
        self._replacement_combo.set_text_column(0)

    def _init_sensitivities(self):
        """Initialize widget sensitivities."""
        self._next_button.set_sensitive(False)
        self._pattern_entry.emit("changed")
        self._previous_button.set_sensitive(False)
        self._regex_check.emit("toggled")
        self._replace_all_button.set_sensitive(False)
        self._replace_button.set_sensitive(False)
        self._replacement_entry.emit("changed")
        self._text_view.set_sensitive(False)

    def _init_signal_handlers(self):
        """Initialize signal handlers."""
        aeidon.util.connect(self, "_pattern_entry", "changed")
        aeidon.util.connect(self, "application", "page-changed")
        callback = lambda *args: args[-1]._update_search_targets()
        self.application.connect("page-added", callback, self)
        gaupol.conf.search.connect("notify::fields", callback, self)
        gaupol.conf.search.connect("notify::target", callback, self)

    def _init_text_view(self):
        """Initialize the text view."""
        gaupol.util.prepare_text_view(self._text_view)
        gaupol.util.scale_to_size(self._text_view, 70, 5)
        text_buffer = self._text_view.get_buffer()
        text_buffer.connect("changed", self._on_text_buffer_changed)

    def _init_values(self):
        """Initialize default values for widgets."""
        self._pattern_entry.set_text(self.application.pattern)
        self._replacement_entry.set_text(self.application.replacement)
        self._regex_check.set_active(gaupol.conf.search.regex)
        self._ignore_case_check.set_active(gaupol.conf.search.ignore_case)
        fields = gaupol.conf.search.fields
        self._main_check.set_active(gaupol.fields.MAIN_TEXT in fields)
        self._tran_check.set_active(gaupol.fields.TRAN_TEXT in fields)
        target = gaupol.conf.search.target
        self._all_radio.set_active(target == gaupol.targets.ALL)
        self._current_radio.set_active(target == gaupol.targets.CURRENT)
        self._update_search_targets()

    def _on_all_radio_toggled(self, radio_button):
        """Save search target."""
        gaupol.conf.search.target = self._get_target()

    def _on_application_page_changed(self, application, page):
        """Invalidate the current search if underlying data has changed."""
        # If data in page was changed from outside the search dialog, the
        # current search must be invalidated to avoid making edits (especially
        # via the text view's focus-out handler) based on data that no longer
        # exists. All search dialog's data changing methods should be wrapped
        # to disable this handling of application's page-changed signals.
        if self._handle_page_changes and (self._match_page is not None):
            self._reset_properties()

    def _on_current_radio_toggled(self, radio_button):
        """Save search target."""
        gaupol.conf.search.target = self._get_target()

    def _on_ignore_case_check_toggled(self, check_button):
        """Save ignore case setting."""
        gaupol.conf.search.ignore_case = check_button.get_active()

    def _on_main_check_toggled(self, check_button):
        """Save search target."""
        gaupol.conf.search.fields = self._get_fields()
        self._search_vbox.set_sensitive(bool(self._get_fields()))

    def _on_next_button_clicked(self, *args):
        """Find the next match of pattern."""
        # Make sure the text view experiences a focus-out-event.
        self._next_button.grab_focus()
        self.next()

    def _on_pattern_entry_changed(self, entry):
        """Update action sensitivities."""
        have_pattern = bool(entry.get_text())
        self._next_button.set_sensitive(have_pattern)
        self._previous_button.set_sensitive(have_pattern)
        self._replace_all_button.set_sensitive(have_pattern)

    def _on_previous_button_clicked(self, *args):
        """Find the previous match of pattern."""
        # Make sure the text view experiences a focus-out-event.
        self._previous_button.grab_focus()
        self.previous()

    def _on_regex_check_toggled(self, check_button):
        """Save regular expression setting."""
        use_regex = check_button.get_active()
        gaupol.conf.search.regex = use_regex
        self.set_response_sensitive(gtk.RESPONSE_HELP, use_regex)

    def _on_replace_all_button_clicked(self, *args):
        """Replace all matches of pattern."""
        # Make sure the text view experiences a focus-out-event.
        self._replace_all_button.grab_focus()
        gaupol.util.set_cursor_busy(self._dialog)
        self.replace_all()
        gaupol.util.set_cursor_normal(self._dialog)

    def _on_replace_button_clicked(self, *args):
        """Replace the current match of pattern."""
        # Make sure the text view experiences a focus-out-event.
        self._replace_button.grab_focus()
        self._replace_button.set_sensitive(False)
        self.replace()

    def _on_response(self, dialog, response):
        """Do not send response if browsing help."""
        if response == gtk.RESPONSE_HELP:
            gaupol.util.show_uri(gaupol.REGEX_HELP_URL)
            self.stop_emission("response")

    def _on_show(self, *args):
        """Move focus to the pattern entry and select the pattern."""
        self._pattern_entry.select_region(0, -1)
        self._pattern_entry.grab_focus()

    def _on_text_buffer_changed(self, *args):
        """Disable replace action."""
        self._replace_button.set_sensitive(False)

    @page_changing
    def _on_text_view_focus_out_event(self, text_view, event):
        """Save changes made in `text_view`."""
        if self._match_page is None: return
        if self._match_row  is None: return
        if self._match_doc  is None: return
        if not self._text_view.props.sensitive: return
        page = self.application.get_current_page()
        if page is not self._match_page: return
        text_buffer = text_view.get_buffer()
        bounds = text_buffer.get_bounds()
        text = text_buffer.get_text(*bounds)
        page.project.set_text(self._match_row, self._match_doc, text)

    def _on_tran_check_toggled(self, check_button):
        """Save search target."""
        gaupol.conf.search.fields = self._get_fields()
        self._search_vbox.set_sensitive(bool(self._get_fields()))

    def _read_history(self, name):
        """Read history from file of type `name`."""
        directory = os.path.join(aeidon.CONFIG_HOME_DIR, "search")
        path = os.path.join(directory, "%s.history" % name)
        if not os.path.isfile(path): return
        history = aeidon.util.readlines(path)
        setattr(self, name, history)

    def _reset_properties(self):
        """Reset search properties to defaults."""
        self._match_page = None
        self._match_doc  = None
        self._match_row  = None
        self._match_span = None
        self._text_view.get_buffer().set_text("")
        self._text_view.set_sensitive(False)

    def _set_failure_state(self):
        """Set search properties for failure to find a match."""
        self._reset_properties()
        pattern = self._pattern_entry.get_text()
        message = _('"%s" not found') % pattern
        self.application.flash_message(message)

    def _set_pattern(self, page):
        """Set search pattern for `page`.

        Raise :exc:`gaupol.Default` if pattern empty or bad.
        """
        pattern = self._pattern_entry.get_text()
        gaupol.util.raise_default(not pattern)
        ignore_case = gaupol.conf.search.ignore_case
        if not gaupol.conf.search.regex:
            self._add_pattern_to_history()
            return page.project.set_search_string(pattern, ignore_case)
        flags = (re.IGNORECASE if ignore_case else 0)
        try: page.project.set_search_regex(pattern, flags)
        except re.error as message:
            self._show_regex_error_dialog_pattern(str(message))
            raise gaupol.Default
        self._add_pattern_to_history()

    def _set_replacement(self, page):
        """Set search replacement for `page`."""
        replacement = self._replacement_entry.get_text()
        page.project.set_search_replacement(replacement)
        self._add_replacement_to_history()

    def _set_success_state(self, page, row, doc, match_span, next):
        """Set search properties for found match."""
        self._match_page = page
        self._match_row = row
        self._match_doc = doc
        self._match_span = list(match_span)
        self._was_next = next
        self.application.set_current_page(page)
        self._set_text(page, row, doc, match_span)
        col = page.document_to_text_column(doc)
        page.view.set_focus(row, col)
        page.view.scroll_to_row(row)
        self._replace_button.set_sensitive(True)

    def _set_text(self, page, row, doc, match_span):
        """Set subtitle text to text view."""
        text_buffer = self._text_view.get_buffer()
        subtitle = page.project.subtitles[row]
        text = subtitle.get_text(doc)
        text_buffer.set_text(text)
        ins = text_buffer.get_iter_at_offset(match_span[0])
        bound = text_buffer.get_iter_at_offset(match_span[1])
        text_buffer.select_range(ins, bound)
        mark = text_buffer.create_mark(None, bound, True)
        self._text_view.scroll_to_mark(mark, 0.2)
        self._text_view.set_sensitive(True)
        self._text_view.grab_focus()

    def _show_regex_error_dialog_pattern(self, message):
        """Show an error dialog if regex pattern failed to compile."""
        title = _('Error in regular expression pattern')
        dialog = gaupol.ErrorDialog(self._dialog, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _show_regex_error_dialog_replacement(self, message):
        """Show an error dialog if regex replacement is invalid."""
        title = _('Error in regular expression replacement')
        dialog = gaupol.ErrorDialog(self._dialog, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _update_search_targets(self):
        """Update search targets in all pages."""
        docs = map(gaupol.util.text_field_to_document,
                   gaupol.conf.search.fields)

        wrap = (gaupol.conf.search.target != gaupol.targets.ALL)
        for page in self.application.pages:
            page.project.set_search_target(None, docs, wrap)

    def _write_history(self, name):
        """Write history to file of type `name`."""
        directory = os.path.join(aeidon.CONFIG_HOME_DIR, "search")
        try: aeidon.util.makedirs(directory)
        except OSError: return
        path = os.path.join(directory, "%s.history" % name)
        history = getattr(self, name)
        text = os.linesep.join(history) + os.linesep
        aeidon.util.write(path, text)

    def next_require(self):
        assert self._pattern_entry.get_text()

    @aeidon.deco.silent(gaupol.Default)
    def next(self):
        """Find the next match of pattern."""
        page = self.application.get_current_page()
        row, doc, pos = self._get_position(True)
        pages = self.application.pages[:]
        i = pages.index(page)
        # Loop twice over the current page,
        # since starting in the middle.
        for page in (pages[i:] + pages[:i + 1]):
            self._set_pattern(page)
            try:
                # Return match in page after position.
                row, doc, span = page.project.find_next(row, doc, pos)
                return self._set_success_state(page, row, doc, span, True)
            except StopIteration:
                target = gaupol.conf.search.target
                if target == gaupol.targets.CURRENT:
                    # Fail wrapped single-page search.
                    return self._set_failure_state()
                # Proceed to the next page.
                row = doc = pos = None
        # Fail wrapped search of all pages.
        self._set_failure_state()

    def previous_require(self):
        assert self._pattern_entry.get_text()

    @aeidon.deco.silent(gaupol.Default)
    def previous(self):
        """Find the previous match of pattern."""
        page = self.application.get_current_page()
        row, doc, pos = self._get_position(False)
        pages = self.application.pages[::-1]
        i = pages.index(page)
        # Loop twice over the current page,
        # since starting in the middle.
        for page in (pages[i:] + pages[:i + 1]):
            self._set_pattern(page)
            try:
                # Return match in page before position.
                row, doc, span = page.project.find_previous(row, doc, pos)
                return self._set_success_state(page, row, doc, span, False)
            except StopIteration:
                target = gaupol.conf.search.target
                if target == gaupol.targets.CURRENT:
                    # Fail wrapped single-page search.
                    return self._set_failure_state()
                # Proceed to the previous page.
                row = doc = pos = None
        # Fail wrapped search of all pages.
        self._set_failure_state()

    def replace_require(self):
        assert self._match_doc  is not None
        assert self._match_page is not None
        assert self._match_row  is not None
        assert self._match_span is not None
        assert self._was_next   is not None

    @page_changing
    @aeidon.deco.silent(gaupol.Default)
    def replace(self):
        """Replace the current match of pattern."""
        page = self.application.get_current_page()
        if page is not self._match_page: return
        self._set_replacement(page)
        subtitle = page.project.subtitles[self._match_row]
        length = len(subtitle.get_text(self._match_doc))
        try: page.project.replace()
        except re.error as message:
            return self._show_regex_error_dialog_replacement(str(message))
        shift = (len(subtitle.get_text(self._match_doc)) - length)
        self._match_span[1] += shift
        (self.next if self._was_next else self.previous)()

    def replace_all_require(self):
        assert self._pattern_entry.get_text()

    @page_changing
    @aeidon.deco.silent(gaupol.Default)
    def replace_all(self):
        """Replace all matches of pattern."""
        count = 0
        target = gaupol.conf.search.target
        for page in self.application.get_target_pages(target):
            self._set_pattern(page)
            self._set_replacement(page)
            try: count += page.project.replace_all()
            except re.error as message:
                self._show_regex_error_dialog_replacement(str(message))
                break
        self._reset_properties()
        self.application.flash_message(aeidon.i18n.ngettext(
                "Found and replaced %d occurence",
                "Found and replaced %d occurences",
                count) % count)



#!/usr/bin/env python

# Copyright (C) 2009 Osmo Salomaa
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Walk module sources and autogenerate RST documentation files.

Usage: python autogen.py MODULE_NAME...

Requires RST template files 'class.rst.in', 'index.rst.in', 'module.rst.in' and
'source.rst.in' somewhere in templates_path as defined in conf.py. Suggested to
be used with the 'autoclean' extension. Together, this script, the templates
and the 'autoclean' extension should provide decent and complete API
documentation automatically generated from source code with verbose docstrings
written in reStructuredText markup. Additional hand-written documentation can
be added by editing the 'index.rst.in' template to include introductory text or
links to the separate, handwritten files.
"""

import codecs
import imp
import inspect
import jinja2
import os
import sys

conf = os.path.join(os.path.dirname(__file__), "conf.py")
conf = imp.load_source("conf", conf)


def document_class(names, cls):
    """Write RST-file documenting class and its members."""
    print ".".join(names)
    write_template("class",
                   names,
                   anchestors=get_anchestors(cls),
                   cls=".".join(names),
                   cls_short=names[-1],
                   is_exception=issubclass(cls, Exception),
                   methods=get_methods(cls),
                   module=".".join(names[:-1]),
                   source_doc=get_source_doc(cls),
                   source_fname=get_source_fname(cls))

def document_module(names, module):
    """Write RST-file documenting module and its members."""
    print ".".join(names)
    document_source(names, module)
    write_template("module",
                   names,
                   classes=get_classes(module),
                   functions=get_functions(module),
                   module=".".join(names),
                   modules=get_modules(module),
                   root=conf.autogen_output_path,
                   source_doc=get_source_doc(module),
                   source_fname=get_source_fname(module))

    for name in get_modules(module):
        document_module(tuple(names) + (name,),
                        getattr(module, name))

    for name in get_classes(module):
        document_class(tuple(names) + (name,),
                       getattr(module, name))

def document_source(names, module):
    """Write RST-file with module source code."""
    names_out = list(names)
    names_out[-1] = "%s_source" % names_out[-1]
    write_template("source",
                   names_out,
                   module=".".join(names),
                   source_fname=get_source_fname(module),
                   source_include=get_source_include(module))

def filter_members(members):
    """Return subset of members to appear in documentation."""
    if not conf.private_members:
        members = [x for x in members
                   if (not x.startswith("_") or
                       x in conf.include_members)]
    members = set(members) - set(conf.exclude_members)
    members = [x for x in members
               if not x.endswith(tuple(conf.exclude_members_endswith))]
    return sorted(list(members))

def get_anchestors(cls):
    """Return a list of names of anchestor classes."""
    if not cls.__bases__:
        return []
    base = cls.__bases__[0]
    name = "%s.%s" % (base.__module__, base.__name__)
    name = name.replace("__builtin__.", "")
    return [name] + get_anchestors(base)

def get_classes(module):
    """Return list of names of classes defined in module."""
    is_class = lambda x: inspect.isclass(getattr(module, x))
    classes = filter(is_class, module.__dict__.keys())
    top = module.__name__.split(".")[0]
    for i in reversed(range(len(classes))):
        candidate = getattr(module, classes[i])
        parent = inspect.getmodule(candidate)
        if (parent is None or
            not parent.__name__.startswith(top)):
            classes.pop(i)
    return filter_members(classes)

def get_functions(module):
    """Return list of names of functions defined in module."""
    is_function = lambda x: inspect.isfunction(getattr(module, x))
    functions = filter(is_function, module.__dict__.keys())
    top = module.__name__.split(".")[0]
    for i in reversed(range(len(functions))):
        candidate = getattr(module, functions[i])
        parent = inspect.getmodule(candidate)
        if (parent is None or
            not parent.__name__.startswith(top)):
            functions.pop(i)
    return filter_members(functions)

def get_methods(cls):
    """Return list of names of methods defined in cls."""
    is_method = lambda x: inspect.ismethod(getattr(cls, x))
    methods = filter(is_method, cls.__dict__.keys())
    return filter_members(methods)

def get_modules(module):
    """Return list of names of modules defined in module."""
    is_module = lambda x: inspect.ismodule(getattr(module, x))
    modules = filter(is_module, module.__dict__.keys())
    top = module.__name__.split(".")[0]
    for i in reversed(range(len(modules))):
        candidate = getattr(module, modules[i])
        parent = inspect.getmodule(candidate)
        if (parent is None or
            module.__name__.startswith(parent.__name__) or
            not parent.__name__.startswith(top)):
            modules.pop(i)
    return filter_members(modules)

def get_source_doc(obj):
    """Return path to obj source RST document or None."""
    if get_source_path(obj) is None:
        return None
    dotted_name = None
    if inspect.ismodule(obj):
        dotted_name = obj.__name__
    if inspect.isclass(obj):
        dotted_name = obj.__module__
    if dotted_name is None:
        return None
    return "/%s/%s_source" % (conf.autogen_output_path, dotted_name)

def get_source_fname(obj):
    """Return obj filename relative to project root or None."""
    path = get_source_path(obj)
    if path is None:
        return None
    directory = os.path.dirname(__file__)
    root = os.path.join(directory, conf.project_root)
    root = os.path.abspath(root)
    fname = path.replace(root, "")
    while fname.startswith(os.sep):
        fname = fname[1:]
    return fname

def get_source_include(obj):
    """Return obj filename relative to documentation root or None."""
    fname = get_source_fname(obj)
    if fname is None:
        return None
    return "/%s" % os.path.join(conf.project_root, fname)

def get_source_path(obj):
    """Return absolute path to file obj is defined in or None."""
    try:
        path = inspect.getfile(obj)
    except TypeError:
        return None
    if path.endswith(("pyc", "pyo")):
        path = path[:-1]
    return os.path.abspath(path)

def main(modules):
    """Recursively document all modules."""
    write_template("index",
                   ("index",),
                   project=conf.project,
                   project_url=conf.project_url,
                   modules=modules,
                   root=conf.autogen_output_path)

    for module in modules:
        document_module(module.split("."),
                        __import__(module))

def write_template(name_in, names_out, **kwargs):
    """Write RST-template to file based in values in kwargs."""
    directory = os.path.dirname(__file__)
    basename = "%s.rst.in" % name_in
    for template_dir in conf.templates_path:
        template_dir = os.path.join(directory, template_dir)
        template_file = os.path.join(template_dir, basename)
        if os.path.isfile(template_file): break
    text = codecs.open(template_file, "r", conf.source_encoding).read()
    template = jinja2.Template(text)
    for key, value in kwargs.items():
        underline = "%s_double_underline" % key
        kwargs[underline] = "=" * len(str(value))
        underline = "%s_single_underline" % key
        kwargs[underline] = "-" * len(str(value))
    text = template.render(**kwargs)
    if not text.endswith(("\n", "\r")):
        text = "%s%s" % (text, os.linesep)
    if name_in == "index":
        return codecs.open("index.rst", "w", conf.source_encoding).write(text)
    names_out = list(names_out)
    names_out[-1] = "%s.rst" % names_out[-1]
    path = os.path.join(directory,
                        conf.autogen_output_path,
                        ".".join(names_out))

    if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
    codecs.open(path, "w", conf.source_encoding).write(text)


if __name__ == "__main__":
    main(sys.argv[1:])



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class _TestPositionShiftDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        self.application = self.new_application()
        page = self.application.get_current_page()
        page.view.select_rows((1, 2, 3))
        gaupol.conf.preview.use_custom_command = True
        gaupol.conf.preview.custom_command = "echo"
        page.project.video_path = self.new_subrip_file()

    def test__init____no(self):
        # pylint: disable=W0201
        page = self.application.get_current_page()
        page.project.video_path = None
        page.project.main_file = None
        page.view.select_rows(())
        gaupol.conf.position_shift.target = gaupol.targets.SELECTED
        self.dialog = self.dialog.__class__(self.application.window,
                                            self.application)

    def test__on_amount_spin_value_changed(self):
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_BACKWARD)
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_BACKWARD)

    def test__on_preview_button_clicked__0(self):
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog._current_radio.set_active(True)
        self.dialog._preview_button.emit("clicked")

    def test__on_preview_button_clicked__selection(self):
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog._selected_radio.set_active(True)
        self.dialog._preview_button.emit("clicked")

    def test__on_response__all(self):
        self.dialog._all_radio.set_active(True)
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog.response(gtk.RESPONSE_OK)

    def test__on_response__current(self):
        self.dialog._current_radio.set_active(True)
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog.response(gtk.RESPONSE_OK)

    def test__on_response__selected(self):
        self.dialog._selected_radio.set_active(True)
        self.dialog._amount_spin.spin(gtk.SPIN_STEP_FORWARD)
        self.dialog.response(gtk.RESPONSE_OK)


class TestFrameShiftDialog(_TestPositionShiftDialog):

    def setup_method(self, method):
        _TestPositionShiftDialog.setup_method(self, method)
        self.dialog = gaupol.FrameShiftDialog(self.application.window,
                                              self.application)

        self.dialog.show()


class TestTimeShiftDialog(_TestPositionShiftDialog):

    def setup_method(self, method):
        _TestPositionShiftDialog.setup_method(self, method)
        self.dialog = gaupol.TimeShiftDialog(self.application.window,
                                             self.application)

        self.dialog.show()



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for objects that dispatch ``self``-lookups."""

__all__ = ("Delegate",)


class Delegate(object):

    """Base class for objects that dispatch ``self``-lookups.

    :ivar master: Object to where attribute calls are dispatched
    """

    def __getattr__(self, name):
        """Return value of master attribute."""
        return getattr(self.master, name)

    def __init__(self, master):
        """Initialize a :class:`Delegate` object."""
        object.__setattr__(self, "master", master)

    def __setattr__(self, name, value):
        """Set value of master attribute."""
        # Do not create new attributes for master.
        if hasattr(self.master, name):
            return setattr(self.master, name, value)
        return object.__setattr__(self, name, value)

    def _invariant(self):
        # Default to checking master's class invariant.
        if hasattr(self.master, "_invariant"):
            return self.master._invariant()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Basic editing of entire subtitles."""

from __future__ import division

import aeidon
_ = aeidon.i18n._


class EditAgent(aeidon.Delegate):

    """Basic editing of entire subtitles."""

    __metaclass__ = aeidon.Contractual

    @aeidon.deco.revertable
    @aeidon.deco.notify_frozen
    def _insert_blank_subtitles(self, indices, register=-1):
        """Insert new blank subtitles.

        Set sensible equal duration positions within given window or with 3
        second durations if window not limited.
        """
        for rindices in aeidon.util.get_ranges(indices):
            first_start = 0.0
            if self.subtitles and (rindices[0] == 0):
                start = self.subtitles[0].start_seconds
                first_start = (0.0 if start >= 0 else start - 3.0)
            if rindices[0] > 0:
                subtitle = self.subtitles[rindices[0] - 1]
                first_start = subtitle.end_seconds
            duration = 3.0
            if rindices[0] < len(self.subtitles):
                subtitle = self.subtitles[rindices[0]]
                window = subtitle.start_seconds - first_start
                duration = window / len(rindices)
            for i, index in enumerate(rindices):
                subtitle = self.new_subtitle()
                subtitle.start = first_start + (i * duration)
                subtitle.duration = duration
                self.subtitles.insert(index, subtitle)
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Inserting subtitles")
        action.revert_function = self.remove_subtitles
        action.revert_args = (indices,)
        self.register_action(action)
        self.emit("subtitles-inserted", indices)

    def clear_texts_require(self, indices, doc, register=-1):
        for index in indices:
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def clear_texts(self, indices, doc, register=-1):
        """Set texts to blank strings."""
        new_texts =  [""] * len(indices)
        self.replace_texts(indices, doc, new_texts, register=register)
        self.set_action_description(register, _("Clearing texts"))

    @aeidon.deco.export
    @aeidon.deco.revertable
    @aeidon.deco.notify_frozen
    def insert_subtitles(self, indices, subtitles=None, register=-1):
        """Insert `subtitles` at indices.

        If `subtitles` is None, insert new blank subtitles at indices with
        sensible equal duration positions within given window or with 3 second
        durations if window not limited.
        """
        if subtitles is None:
            return self._insert_blank_subtitles(indices, register=register)
        for i, index in enumerate(indices):
            self.subtitles.insert(index, subtitles[i])
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Inserting subtitles")
        action.revert_function = self.remove_subtitles
        action.revert_args = (indices,)
        self.register_action(action)
        self.emit("subtitles-inserted", indices)

    def merge_subtitles_require(self, indices, register=-1):
        assert len(indices) > 1
        for index in indices:
            assert 0 <= index < len(self.subtitles)
        assert list(indices) == range(indices[0], indices[-1] + 1)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def merge_subtitles(self, indices, register=-1):
        """Merge subtitles at `indices` to form one subtitle."""
        subtitle = self.new_subtitle()
        subtitle.start = self.subtitles[indices[0]].start
        subtitle.end = self.subtitles[indices[-1]].end
        main_texts = [self.subtitles[i].main_text for i in indices]
        subtitle.main_text = "\n".join(filter(None, main_texts))
        tran_texts = [self.subtitles[x].tran_text for x in indices]
        subtitle.tran_text = "\n".join(filter(None, tran_texts))
        self.remove_subtitles(indices, register=register)
        self.insert_subtitles([indices[0]], [subtitle], register=register)
        self.group_actions(register, 2, _("Merging subtitles"))

    def remove_subtitles_require(self, indices, register=-1):
        for index in indices:
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    @aeidon.deco.notify_frozen
    def remove_subtitles(self, indices, register=-1):
        """Remove subtitles at `indices`."""
        indices = sorted(indices)
        subtitles = list(reversed([self.subtitles.pop(i)
                                   for i in reversed(indices)]))

        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Removing subtitles")
        action.revert_function = self.insert_subtitles
        action.revert_args = (indices, subtitles)
        self.register_action(action)
        self.emit("subtitles-removed", indices)

    def replace_positions_require(self, indices, subtitles, register=-1):
        for index in indices:
            assert 0 <= index < len(self.subtitles)
        assert len(indices) == len(subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    @aeidon.deco.notify_frozen
    def replace_positions(self, indices, subtitles, register=-1):
        """Replace positions at `indices` with those from `subtitles`."""
        orig_subtitles = [self.subtitles[i].copy() for i in indices]
        for i, index in enumerate(indices):
            self.subtitles[index].start = subtitles[i].start
            self.subtitles[index].end = subtitles[i].end
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Replacing positions")
        action.revert_function = self.replace_positions
        action.revert_args = (indices, orig_subtitles)
        self.register_action(action)
        self.emit("positions-changed", indices)

    def replace_texts_require(self, indices, doc, texts, register=-1):
        for index in indices:
            assert 0 <= index < len(self.subtitles)
        assert len(indices) == len(texts)

    @aeidon.deco.export
    @aeidon.deco.revertable
    @aeidon.deco.notify_frozen
    def replace_texts(self, indices, doc, texts, register=-1):
        """Replace texts in `doc`'s `indices` with `texts`."""
        orig_texts = [self.subtitles[i].get_text(doc) for i in indices]
        for i, index in enumerate(indices):
            self.subtitles[index].set_text(doc, texts[i])
        action = self.new_revertable_action(register)
        action.docs = (doc,)
        action.description = _("Replacing texts")
        action.revert_function = self.replace_texts
        action.revert_args = (indices, doc, orig_texts)
        self.register_action(action)
        self.emit(self.get_text_signal(doc), indices)

    def split_subtitle_require(self, index, register=-1):
        assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def split_subtitle(self, index, register=-1):
        """Split subtitle in two equal duration ones."""
        middle = self.calc.get_middle(self.subtitles[index].start,
                                      self.subtitles[index].end)

        subtitle_1 = self.new_subtitle()
        subtitle_1.start = self.subtitles[index].start
        subtitle_1.end = middle
        subtitle_1.main_text = self.subtitles[index].main_text
        subtitle_1.tran_text = self.subtitles[index].tran_text
        subtitle_2 = self.new_subtitle()
        subtitle_2.start = middle
        subtitle_2.end = self.subtitles[index].end
        self.remove_subtitles([index], register=register)
        self.insert_subtitles((index, index + 1),
                              (subtitle_1, subtitle_2),
                              register=register)

        self.group_actions(register, 2, _("Splitting subtitle"))



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestFormatAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    def test_change_case__dialogue(self):
        self.project.subtitles[0].main_text = (
            "- mrs. pavinato?\n"
            "- yes, what do you want?")
        self.project.change_case((0,), aeidon.documents.MAIN, "title")
        assert self.project.subtitles[0].main_text == (
            "- Mrs. Pavinato?\n"
            "- Yes, What Do You Want?")

    def test_change_case__italic(self):
        self.project.subtitles[0].main_text = (
            "<i>mrs. pavinato?</i>\n"
            "<i>yes, what do you want?</i>")
        self.project.change_case((0,), aeidon.documents.MAIN, "capitalize")
        assert self.project.subtitles[0].main_text == (
            "<i>Mrs. pavinato?</i>\n"
            "<i>yes, what do you want?</i>")

    def test_change_case__plain(self):
        self.project.subtitles[0].main_text = (
            "mrs. pavinato?\n"
            "yes, what do you want?")
        self.project.change_case((0,), aeidon.documents.MAIN, "upper")
        assert self.project.subtitles[0].main_text == (
            "MRS. PAVINATO?\n"
            "YES, WHAT DO YOU WANT?")

    def test_add_dialogue_dashes(self):
        self.project.subtitles[0].main_text = (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        self.project.subtitles[1].main_text = (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")
        self.project.add_dialogue_dashes((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        assert self.project.subtitles[1].main_text == (
            "- It was the only beautiful thing you had.\n"
            "- Now you seem a different person.")

    def test_italicize(self):
        self.project.subtitles[0].main_text = (
            "<i>I am no thief, I am an officer\n"
            "and a university student.</i>")
        self.project.subtitles[1].main_text = (
            "I look like this because\n"
            "I'm hunted for by the Germans.")
        self.project.italicize((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "<i>I am no thief, I am an officer\n"
            "and a university student.</i>")
        assert self.project.subtitles[1].main_text == (
            "<i>I look like this because\n"
            "I'm hunted for by the Germans.</i>")

    def test_remove_dialogue_dashes(self):
        self.project.subtitles[0].main_text = (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        self.project.subtitles[1].main_text = (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")
        self.project.remove_dialogue_dashes((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "You have cut your beard?\n"
            "Yes, don't you like it?")
        assert self.project.subtitles[1].main_text == (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")

    def test_toggle_dialogue_dashes__all(self):
        self.project.subtitles[0].main_text = (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        self.project.subtitles[1].main_text = (
            "- It was the only beautiful thing you had.\n"
            "- Now you seem a different person.")
        self.project.toggle_dialogue_dashes((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "You have cut your beard?\n"
            "Yes, don't you like it?")
        assert self.project.subtitles[1].main_text == (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")

    def test_toggle_dialogue_dashes__none(self):
        self.project.subtitles[0].main_text = (
            "You have cut your beard?\n"
            "Yes, don't you like it?")
        self.project.subtitles[1].main_text = (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")
        self.project.toggle_dialogue_dashes((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        assert self.project.subtitles[1].main_text == (
            "- It was the only beautiful thing you had.\n"
            "- Now you seem a different person.")

    def test_toggle_dialogue_dashes__some(self):
        self.project.subtitles[0].main_text = (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        self.project.subtitles[1].main_text = (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")
        self.project.toggle_dialogue_dashes((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        assert self.project.subtitles[1].main_text == (
            "- It was the only beautiful thing you had.\n"
            "- Now you seem a different person.")

    def test_toggle_italicization__all(self):
        self.project.subtitles[0].main_text = (
            "<b><i>I am no thief, I am an officer\n"
            "and a university student.</i></b>")
        self.project.subtitles[1].main_text = (
            "<i><b>I look like this because\n"
            "I'm hunted for by the Germans.</b></i>")
        self.project.toggle_italicization((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "<b>I am no thief, I am an officer\n"
            "and a university student.</b>")
        assert self.project.subtitles[1].main_text == (
            "<b>I look like this because\n"
            "I'm hunted for by the Germans.</b>")

    def test_toggle_italicization__none(self):
        self.project.subtitles[0].main_text = (
            "I am no thief, I am an officer\n"
            "and a university student.")
        self.project.subtitles[1].main_text = (
            "I look like this because\n"
            "I'm hunted for by the Germans.")
        self.project.toggle_italicization((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "<i>I am no thief, I am an officer\n"
            "and a university student.</i>")
        assert self.project.subtitles[1].main_text == (
            "<i>I look like this because\n"
            "I'm hunted for by the Germans.</i>")

    def test_toggle_italicization__some(self):
        self.project.subtitles[0].main_text = (
            "<i>I am no thief, I am an officer\n"
            "and a university student.</i>")
        self.project.subtitles[1].main_text = (
            "I look like this because\n"
            "I'm hunted for by the Germans.")
        self.project.toggle_italicization((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "<i>I am no thief, I am an officer\n"
            "and a university student.</i>")
        assert self.project.subtitles[1].main_text == (
            "<i>I look like this because\n"
            "I'm hunted for by the Germans.</i>")

    def test_remove_dialogue_dashes__all(self):
        self.project.subtitles[0].main_text = (
            "- You have cut your beard?\n"
            "- Yes, don't you like it?")
        self.project.subtitles[1].main_text = (
            "- It was the only beautiful thing you had.\n"
            "- Now you seem a different person.")
        self.project.remove_dialogue_dashes((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "You have cut your beard?\n"
            "Yes, don't you like it?")
        assert self.project.subtitles[1].main_text == (
            "It was the only beautiful thing you had.\n"
            "Now you seem a different person.")

    def test_unitalicize(self):
        self.project.subtitles[0].main_text = (
            "<i>I am no thief, I am an officer\n"
            "and a university student.</i>")
        self.project.subtitles[1].main_text = (
            "I look like this because\n"
            "I'm hunted for by the Germans.")
        self.project.unitalicize((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == (
            "I am no thief, I am an officer\n"
            "and a university student.")
        assert self.project.subtitles[1].main_text == (
            "I look like this because\n"
            "I'm hunted for by the Germans.")



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestMPsubTime(aeidon.TestCase):

    format = aeidon.formats.MPSUB
    name = "mpsub-time"

    def setup_method(self, method):
        path = self.new_temp_file(self.format, self.name)
        self.file = aeidon.files.new(self.format, path, "ascii")

    def test_copy_from(self):
        self.file.set_header("FORMAT=25.00")
        path = self.new_temp_file(self.format, self.name)
        new_file = aeidon.files.new(self.format, path, "ascii")
        new_file.copy_from(self.file)
        assert new_file.header == self.file.header
        assert new_file.framerate == self.file.framerate
        assert new_file.mode == self.file.mode

    def test_read(self):
        assert self.file.read()
        assert self.file.header
        assert self.file.mode in aeidon.modes
        if self.file.framerate is not None:
            assert self.file.framerate in aeidon.framerates

    def test_set_header__frame(self):
        self.file.set_header("FORMAT=29.97")
        assert self.file.mode == aeidon.modes.FRAME
        assert self.file.framerate == aeidon.framerates.FPS_29_970

    def test_set_header__time(self):
        self.file.set_header("FORMAT=TIME")
        assert self.file.mode == aeidon.modes.TIME
        assert self.file.framerate == aeidon.framerates.NONE

    def test_set_header__value_error(self):
        self.raises(ValueError, self.file.set_header, "FORMAT=NONE")

    def test_write(self):
        self.file.write(self.file.read(), aeidon.documents.MAIN)
        text = open(self.file.path, "r").read().strip()
        reference = self.get_sample_text(self.format, self.name)
        assert text == reference


class TestMPsubFrame(TestMPsubTime):

    name = "mpsub-frame"



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon

from .test_ssa import TestSubStationAlpha


class TestAdvSubStationAlpha(TestSubStationAlpha):

    text = ("All things weird are normal\n"
            "in this whore of cities.")

    def setup_method(self, method):
        self.markup = aeidon.tags.new(aeidon.formats.ASS)

    def test_decode__bold_weight(self):
        text = ("All things weird are normal\n"
                "in {\\b500}this{\\b0} whore of cities.")
        assert self.markup.decode(text) == (
            "All things weird are normal\n"
            "in <b>this</b> whore of cities.")

    def test_decode__underline(self):
        text = ("{\\u1}All things weird are normal\n"
                "in this whore of cities{\\rDefault}.")
        assert self.markup.decode(text) == (
            "<u>All things weird are normal\n"
            "in this whore of cities</u>.")

    def test_encode__underline(self):
        text = ("All things weird are normal\n"
                "<u>in this whore of cities.</u>")
        assert self.markup.encode(text) == (
            "All things weird are normal\n"
            "{\\u1}in this whore of cities.{\\u0}")



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""View menu UI manager actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
_ = aeidon.i18n._


class ActivateNextProjectAction(gaupol.Action):

    """Activate the project in the next tab."""

    def __init__(self):
        """Initialize an :class:`ActivateNextProjectAction` object."""
        gaupol.Action.__init__(self, "activate_next_project")
        self.props.label = _("_Next")
        self.props.tooltip = _("Activate the project in the next tab")
        self.accelerator = "<Control>Page_Down"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        index = application.pages.index(page) + 1
        aeidon.util.affirm(index in range(len(application.pages)))


class ActivatePreviousProjectAction(gaupol.Action):

    """Activate the project in the previous tab."""

    def __init__(self):
        """Initialize an :class:`ActivatePreviousProjectAction` object."""
        gaupol.Action.__init__(self, "activate_previous_project")
        self.props.label = _("_Previous")
        self.props.tooltip = _("Activate the project in the previous tab")
        self.accelerator = "<Control>Page_Up"
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(application.pages.index(page) > 0)


class MoveTabLeftAction(gaupol.Action):

    """Move the current tab to the left."""

    def __init__(self):
        """Initialize a :class:`MoveTabLeftAction` object."""
        gaupol.Action.__init__(self, "move_tab_left")
        self.props.label = _("Move Tab _Left")
        self.props.tooltip = _("Move the current tab to the left")
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(application.pages.index(page) > 0)


class MoveTabRightAction(gaupol.Action):

    """Move the current tab to the right."""

    def __init__(self):
        """Initialize a :class:`MoveTabRightAction` object."""
        gaupol.Action.__init__(self, "move_tab_right")
        self.props.label = _("Move Tab _Right")
        self.props.tooltip = _("Move the current tab to the right")
        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        index = application.pages.index(page) + 1
        aeidon.util.affirm(index in range(len(application.pages)))


class ShowColumnsMenuAction(gaupol.MenuAction):

    """Show the "Columns" menu."""
    def __init__(self):
        """Initialize a :class:`ShowColumnsMenuAction` object."""
        gaupol.MenuAction.__init__(self, "show_columns_menu")
        self.props.label = _("_Columns")
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ShowFramerate23976Action(gaupol.RadioAction):

    """Calculate nonnative units with a framerate of 23.976 fps."""

    def __init__(self):
        """Initialize a :class:`ShowFramerate23976Action` object."""
        gaupol.RadioAction.__init__(self, "show_framerate_23_976")
        framerate = gaupol.conf.editor.framerate
        self.props.active = (framerate == aeidon.framerates.FPS_23_976)
        self.props.label = _("2_3.976 fps")
        self.props.tooltip = _("Calculate nonnative units "
                               "with a framerate of 23.976 fps")

        self.props.value = aeidon.framerates.FPS_23_976
        self.action_group = "main-unsafe"
        self.framerate = aeidon.framerates.FPS_23_976
        self.group = "ShowFramerate23976Action"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class ShowFramerate24000Action(gaupol.RadioAction):

    """Calculate nonnative units with a framerate of 24.000 fps."""

    def __init__(self):
        """Initialize a :class:`ShowFramerate24000Action` object."""
        gaupol.RadioAction.__init__(self, "show_framerate_24_000")
        framerate = gaupol.conf.editor.framerate
        self.props.active = (framerate == aeidon.framerates.FPS_24_000)
        self.props.label = _("2_4.000 fps")
        self.props.tooltip = _("Calculate nonnative units "
                               "with a framerate of 24.000 fps")

        self.props.value = aeidon.framerates.FPS_24_000
        self.action_group = "main-unsafe"
        self.framerate = aeidon.framerates.FPS_24_000
        self.group = "ShowFramerate23976Action"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class ShowFramerate25000Action(gaupol.RadioAction):

    """Calculate nonnative units with a framerate of 25.000 fps."""

    def __init__(self):
        """Initialize a :class:`ShowFramerate25000Action` object."""
        gaupol.RadioAction.__init__(self, "show_framerate_25_000")
        framerate = gaupol.conf.editor.framerate
        self.props.active = (framerate == aeidon.framerates.FPS_25_000)
        self.props.label = _("2_5.000 fps")
        self.props.tooltip = _("Calculate nonnative units "
                               "with a framerate of 25.000 fps")

        self.props.value = aeidon.framerates.FPS_25_000
        self.action_group = "main-unsafe"
        self.framerate = aeidon.framerates.FPS_25_000
        self.group = "ShowFramerate23976Action"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class ShowFramerate29970Action(gaupol.RadioAction):

    """Calculate nonnative units with a framerate of 29.970 fps."""

    def __init__(self):
        """Initialize a :class:`ShowFramerate30Action` object."""
        gaupol.RadioAction.__init__(self, "show_framerate_29_970")
        framerate = gaupol.conf.editor.framerate
        self.props.active = (framerate == aeidon.framerates.FPS_29_970)
        self.props.label = _("2_9.970 fps")
        self.props.tooltip = _("Calculate nonnative units "
                               "with a framerate of 29.970 fps")

        self.props.value = aeidon.framerates.FPS_29_970
        self.action_group = "main-unsafe"
        self.framerate = aeidon.framerates.FPS_29_970
        self.group = "ShowFramerate23976Action"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class ShowFramerateMenuAction(gaupol.MenuAction):

    """Show the "Framerate" menu."""

    def __init__(self):
        """Initialize a :class:`ShowFramerateMenuAction` object."""
        gaupol.MenuAction.__init__(self, "show_framerate_menu")
        self.props.label = _("F_ramerate")
        self.action_group = "main-unsafe"
        self.widgets = ("framerate_combo",)

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class ShowFramesAction(gaupol.RadioAction):

    """Show positions as frames."""

    def __init__(self):
        """Initialize a :class:`ShowFramesAction` object."""
        gaupol.RadioAction.__init__(self, "show_frames")
        mode = gaupol.conf.editor.mode
        self.props.active = (mode == aeidon.modes.FRAME)
        self.props.label = _("_Frames")
        self.props.tooltip = _("Show positions as frames")
        self.props.value = aeidon.modes.FRAME
        self.accelerator = "<Shift>T"
        self.action_group = "main-unsafe"
        self.group = "ShowTimesAction"
        self.mode = aeidon.modes.FRAME

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ShowTimesAction(gaupol.RadioAction):

    """Show positions as times."""

    def __init__(self):
        """Initialize a :class:`ShowTimesAction` object."""
        gaupol.RadioAction.__init__(self, "show_times")
        mode = gaupol.conf.editor.mode
        self.props.active = (mode == aeidon.modes.TIME)
        self.props.label = _("_Times")
        self.props.tooltip = _("Show positions as times")
        self.props.value = aeidon.modes.TIME
        self.accelerator = "T"
        self.action_group = "main-unsafe"
        self.group = "ShowTimesAction"
        self.mode = aeidon.modes.TIME

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleDurationColumnAction(gaupol.ToggleAction):

    """Show or hide the duration column."""

    def __init__(self):
        """Initialize a :class:`ToggleDurationColumnAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_duration_column")
        fields = gaupol.conf.editor.visible_fields
        self.props.active = gaupol.fields.DURATION in fields
        self.props.label = _("_Duration")
        self.props.tooltip = _('Show or hide the duration column')
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleEndColumnAction(gaupol.ToggleAction):

    """Show or hide the end column."""

    def __init__(self):
        """Initialize a :class:`ToggleEndColumnAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_end_column")
        fields = gaupol.conf.editor.visible_fields
        self.props.active = gaupol.fields.END in fields
        self.props.label = _("_End")
        self.props.tooltip = _('Show or hide the end column')
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleMainTextColumnAction(gaupol.ToggleAction):

    """Show or hide the main text column."""

    def __init__(self):
        """Initialize a :class:`ToggleMainTextColumnAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_main_text_column")
        fields = gaupol.conf.editor.visible_fields
        self.props.active = gaupol.fields.MAIN_TEXT in fields
        self.props.label = _("_Main Text")
        self.props.tooltip = _('Show or hide the main text column')
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleMainToolbarAction(gaupol.ToggleAction):

    """Show or hide the main toolbar."""

    def __init__(self):
        """Initialize a :class:`ToggleMainToolbarAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_main_toolbar")
        show = gaupol.conf.application_window.show_main_toolbar
        self.props.active = show
        self.props.label = _("_Main Toolbar")
        self.props.tooltip = _("Show or hide the main toolbar")
        self.action_group = "main-safe"


class ToggleNumberColumnAction(gaupol.ToggleAction):

    """Show or hide the number column."""

    def __init__(self):
        """Initialize a :class:`ToggleNumberColumnAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_number_column")
        fields = gaupol.conf.editor.visible_fields
        self.props.active = gaupol.fields.NUMBER in fields
        self.props.label = _("_No.")
        self.props.tooltip = _('Show or hide the number column')
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleOutputWindowAction(gaupol.ToggleAction):

    """Show or hide the output window."""

    def __init__(self):
        """Initialize a :class:`ToggleOutputWindowAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_output_window")
        self.props.active = gaupol.conf.output_window.show
        self.props.label = _("_Output Window")
        self.props.tooltip = _("Show or hide the output window")
        self.action_group = "main-safe"


class ToggleStartColumnAction(gaupol.ToggleAction):

    """Show or hide the start column."""

    def __init__(self):
        """Initialize a :class:`ToggleStartColumnAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_start_column")
        fields = gaupol.conf.editor.visible_fields
        self.props.active = gaupol.fields.START in fields
        self.props.label = _("_Start")
        self.props.tooltip = _('Show or hide the start column')
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleStatusbarAction(gaupol.ToggleAction):

    """Show or hide the statusbar."""

    def __init__(self):
        """Initialize a :class:`ToggleStatusbarAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_statusbar")
        self.props.active = gaupol.conf.application_window.show_statusbar
        self.props.label = _("_Statusbar")
        self.props.tooltip = _("Show or hide the statusbar")
        self.action_group = "main-safe"


class ToggleTranslationTextColumnAction(gaupol.ToggleAction):

    """Show or hide the translation text column."""

    def __init__(self):
        """Initialize a :class:`ToggleTranslationTextColumnAction` object."""
        name = "toggle_translation_text_column"
        gaupol.ToggleAction.__init__(self, name)
        fields = gaupol.conf.editor.visible_fields
        self.props.active = gaupol.fields.TRAN_TEXT in fields
        self.props.label = _("_Translation Text")
        self.props.tooltip = _('Show or hide the translation text column')
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ToggleVideoToolbarAction(gaupol.ToggleAction):

    """Show or hide the video toolbar."""

    def __init__(self):
        """Initialize a :class:`ToggleVideoToolbarAction` object."""
        gaupol.ToggleAction.__init__(self, "toggle_video_toolbar")
        show = gaupol.conf.application_window.show_video_toolbar
        self.props.active = show
        self.props.label = _("_Video Toolbar")
        self.props.tooltip = _("Show or hide the video toolbar")
        self.action_group = "main-safe"


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Metadata store for one item in a desktop-style file."""

import aeidon

__all__ = ("MetadataItem",)


class MetadataItem(object):

    """Metadata store for one item in a desktop-style file.

    :ivar fields: Dictionary mapping field names to their string values

    Common localized fields with custom handling are ``Name`` and
    ``Description``; arbitrary fields are accessible with :meth:`get_field`.
    Strings ``True`` and ``False`` are used for boolean fields.

    For the string syntax and especially the localization handling, see
    freedesktop.org_'s Desktop Entry Specification_.

    .. _freedesktop.org: http://www.freedesktop.org/
    .. _Specification: http://www.freedesktop.org/wiki/Specifications/desktop-entry-spec
    """

    def __init__(self, fields=None):
        """Initialize a :class:`MetadataItem` object."""
        self.fields = fields or {}

    def _get_localized_field(self, name):
        """Return the localized value of field."""
        locale = aeidon.locales.get_system_code()
        modifier = aeidon.locales.get_system_modifier()
        if locale is None:
            return self.get_field(name)
        # 'xx_YY@Zzzz', fall back to 'xx@Zzzz'.
        if ("_" in locale) and (modifier is not None):
            key = "%s[%s@%s]" % (name, locale, modifier)
            if key in self.fields:
                return self.get_field(key)
            locale = locale[0:2]
        # 'xx_YY', fall back to 'xx'.
        if ("_" in locale) and (modifier is None):
            key = "%s[%s]" % (name, locale)
            if key in self.fields:
                return self.get_field(key)
            locale = locale[0:2]
        ## 'xx@Zzzz', fall back to unlocalized.
        if (not "_" in locale) and (modifier is not None):
            key = "%s[%s@%s]" % (name, locale, modifier)
            if key in self.fields:
                return self.get_field(key)
            return self.get_field(name)
        ## 'xx', fall back to unlocalized.
        if (not "_" in locale) and (modifier is None):
            key = "%s[%s]" % (name, locale)
            if key in self.fields:
                return self.get_field(key)
            return self.get_field(name)
        return self.get_field(name)

    def get_description(self, localize=True):
        """Return description as defined by the ``Description`` field."""
        if not localize:
            return self.get_field("Description")
        return self._get_localized_field("Description")

    def get_field(self, name, fallback=None):
        """Return the string value of field or `fallback`."""
        if not name in self.fields:
            return fallback
        return self.fields[name]

    def get_field_boolean(self, name, fallback=None):
        """Return the boolean value of field or `fallback`."""
        if not name in self.fields:
            return fallback
        value = self.fields[name]
        if value == "True":
            return True
        if value == "False":
            return False
        raise ValueError("Invalid boolean value: %s" % repr(value))

    def get_field_list(self, name, fallback=None):
        """Return the list of strings value of field or `fallback`."""
        if not name in self.fields:
            return fallback
        lst = self.fields[name].split(";")
        if not lst[-1]: lst.pop(-1)
        return lst

    def get_name(self, localize=True):
        """Return name as defined by the ``Name`` field."""
        if not localize:
            return self.get_field("Name")
        return self._get_localized_field("Name")

    def has_field(self, name):
        """Return ``True`` if field exists."""
        return (name in self.fields)

    def set_field(self, name, value):
        """Set the string value of field."""
        self.fields[unicode(name)] = unicode(value)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for editing subtitle file headers."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("HeaderDialog",)


class HeaderDialog(gaupol.BuilderDialog):

    """Dialog for editing subtitle file headers."""

    __metaclass__ = aeidon.Contractual

    _widgets = ("copy_down_button",
                "copy_hbox",
                "copy_up_button",
                "main_clear_button",
                "main_revert_button",
                "main_template_button",
                "main_text_view",
                "main_vbox",
                "tran_clear_button",
                "tran_revert_button",
                "tran_template_button",
                "tran_text_view",
                "tran_vbox")

    def __init___require(self, parent, application):
        page = application.get_current_page()
        has_header_count = 0
        if page.project.main_file is not None:
            if page.project.main_file.format.has_header:
                has_header_count += 1
        if page.project.tran_file is not None:
            if page.project.tran_file.format.has_header:
                has_header_count += 1
        assert has_header_count > 0

    def __init__(self, parent, application):
        """Initialize a :class:`HeaderDialog` object."""
        gaupol.BuilderDialog.__init__(self, "header-dialog.ui")
        self.application = application
        gaupol.util.set_widget_font(self._main_text_view, "monospace")
        gaupol.util.set_widget_font(self._tran_text_view, "monospace")
        self._init_values()
        self._init_sizes()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_OK)

    def _get_main_header(self):
        """Return main header from text view."""
        text_buffer = self._main_text_view.get_buffer()
        bounds = text_buffer.get_bounds()
        return text_buffer.get_text(*bounds)

    def _get_translation_header(self):
        """Return translation header from text view."""
        text_buffer = self._tran_text_view.get_buffer()
        bounds = text_buffer.get_bounds()
        return text_buffer.get_text(*bounds)

    def _init_sizes(self):
        """Initialize widget sizes."""
        if self._main_vbox.props.visible:
            gaupol.util.scale_to_content(self._main_text_view,
                                         min_nchar=60,
                                         min_nlines=15,
                                         max_nchar=60,
                                         max_nlines=15,
                                         font="monospace")

        if self._tran_vbox.props.visible:
            gaupol.util.scale_to_content(self._tran_text_view,
                                         min_nchar=60,
                                         min_nlines=15,
                                         max_nchar=60,
                                         max_nlines=15,
                                         font="monospace")

    def _init_values(self):
        """Initialize default values for widgets."""
        main = bool(self._main_file and self._main_file.format.has_header)
        tran = bool(self._tran_file and self._tran_file.format.has_header)
        self._main_vbox.props.visible = main
        self._tran_vbox.props.visible = tran
        self._copy_hbox.props.visible = (main and tran)
        self._set_main_header(self._main_file.header if main else "")
        self._set_translation_header(self._tran_file.header if tran else "")

    @property
    def _main_file(self):
        """Return the main file of the current page's project."""
        page = self.application.get_current_page()
        return page.project.main_file

    def _on_copy_down_button_clicked(self, *args):
        """Copy main header to translation header."""
        self._set_translation_header(self._get_main_header())

    def _on_copy_up_button_clicked(self, *args):
        """Copy translation header to main header."""
        self._set_main_header(self._get_translation_header())

    def _on_main_clear_button_clicked(self, *args):
        """Set a blank string as main header."""
        self._set_main_header("")

    def _on_main_template_button_clicked(self, *args):
        """Set main header to the template of its format."""
        format = self._main_file.format
        self._set_main_header(aeidon.util.get_template_header(format))

    def _on_main_revert_button_clicked(self, *args):
        """Restore original main header."""
        self._set_main_header(self._main_file.header)

    def _on_response(self, dialog, response):
        """Save headers."""
        if response == gtk.RESPONSE_OK:
            self._save_headers()

    def _on_tran_clear_button_clicked(self, *args):
        """Set a blank string as translation header."""
        self._set_translation_header("")

    def _on_tran_template_button_clicked(self, *args):
        """Set translation header to the template of its format."""
        format = self._tran_file.format
        self._set_translation_header(aeidon.util.get_template_header(format))

    def _on_tran_revert_button_clicked(self, *args):
        """Restore original translation header."""
        self._set_translation_header(self._tran_file.header)

    def _save_header(self, sfile, header):
        """Save `header` for `sfile`."""
        if sfile.format == aeidon.formats.MPSUB:
            return self._save_header_mpsub(sfile, header)
        sfile.header = unicode(header)

    def _save_header_mpsub(self, sfile, header):
        """Save `header` for MPsub format `sfile`."""
        try:  sfile.set_header(unicode(header))
        except ValueError:
            return self._show_mpsub_error_dialog()
        if sfile.framerate is None: return
        page = self.application.get_current_page()
        page.project.set_framerate(sfile.framerate, register=None)

    def _save_headers(self):
        """Save headers."""
        if self._main_vbox.props.visible:
            self._save_header(self._main_file,
                              self._get_main_header())

        if self._tran_vbox.props.visible:
            self._save_header(self._tran_file,
                              self._get_translation_header())

        self.application.update_gui()

    def _set_main_header(self, header):
        """Set main `header` to text view."""
        self._main_text_view.get_buffer().set_text(header)

    def _set_translation_header(self, header):
        """Set translation `header` to text view."""
        self._tran_text_view.get_buffer().set_text(header)

    def _show_mpsub_error_dialog(self):
        """Show an error dialog for invalid MPsub header."""
        title = _("Invalid header")
        msg = _('MPsub header must contain a line of form "FORMAT=VALUE", '
                'where VALUE is any of "TIME", "23.98", "25.00" or "29.97".')
        dialog = gaupol.ErrorDialog(self._dialog, title, msg)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    @property
    def _tran_file(self):
        """Return the translation file of the current page's project."""
        page = self.application.get_current_page()
        return page.project.tran_file
